import wx
import sys, os, os.path, time, string, re
import traceback
from threading import Thread
from pieberry_config import *
from pieberry_exec import *
from pieberry_events import *

# pieberry bibdata dicts may contain: 
# author    = *bibtex author string
# title     = *bibtex title string
# year      = *bibtex year
# month     = *bibtex month
# note      = *bibtex note
# annote    = *bibtex annote
# url       = bibtex url
# internal_author = author from file metadata
# internal_title  = title from file metadata
# bibtex_key      = bibtex key (only once saved or specifically set by user)
# bibtex_type     = *bibtex entry type (set to default value for downloads)
# howpublished    = *bibtex howpublished, perhaps with embedded \url or \href
# pdflink         = original link of pdf
# pdflink_context = context string for pdf (e.g. 'submission')
# pdflink_text    = original text string of pdf link
# final_fn        = final resting place of the downloaded file on filesystem
# outfilename     = temporary first location of downloaded file
# exclude         = (bool) should this entry be excluded from the bibliography
# creation_date   = (time_t) creation date of document
# download_time   = (time_t) download time of document
# creation_date_guessed  = (bool) whether the creation date was guessed (t) or gleaned from metadata (f)
# corporate_author       = (bool) is the author in 'author' "corporate" (t) or a person(s) (f)
# ancillary_creationtime = (time_t) deprecated - creation time of document - same as creation_date
# ancillary_outfilename  = deprecated - same as final_fn
# ancillary_locofdoc     = deprecated - same as pdflink
# ancillary_downloadtime = deprecated

class piePrefetchThread(Thread):
    '''thread to prefetch page info'''
    def __init__(self, notify_window, url):
        Thread.__init__(self)
        self._notify_window = notify_window
        self._url = url
        self.start()
    def run(self):
        tag = prefetch_page_info(self._notify_window, self._url)
        if tag:
            wx.PostEvent(self._notify_window, piePrefetchEvent(tag))

class pieWorkerThread(Thread):
    """Worker Thread Class."""
    def __init__(self, notify_window, scrapedata, scrapecommands):
        """Init Worker Thread Class.
        scrapecommands should contain the following:
          - tag_behaviour o(prepend, append_hyphen, append_brackets, dironly)
          - [support recursion some time in future?]
        scrapedata should contain the following:
          - author
          - tag
          - url 
          """
        Thread.__init__(self)
        self._notify_window = notify_window
        self._scrapedata = scrapedata # data from user as necessary
        self._scrapecommands = scrapecommands # directions from user/ui
        self._bibdata = {}
        self._want_abort = 0
        self.start()
 
    def run(self):
        """Run Worker Thread."""
        #TODO - need property to denote if corporate author
        try:
            html = fetch_page(self._notify_window, self._scrapedata['url'])
            pdfs = snarf_pdfs(self._notify_window, html, self._scrapedata['url'])
        except:
            traceback.print_exc()
            # wx.PostEvent(self._notify_window, pieUpdateEvent('fail', 'Failed'))
            wx.PostEvent(self._notify_window, ResultEvent(None))
            return
        curr_pdf_link = 0
        num_pdf_links = len(pdfs)
        re_submission = re.compile(r'^(.+?)(\'s)?( -)? [sS]ubmission')
        return_data = {}
        for pdf in pdfs:
            curr_pdf_link += 1
            if self._want_abort == 1:
                break
            pdf['author'] = self._scrapedata['author']
            pdf['tag'] = self._scrapedata['tag']
            wx.PostEvent(self._notify_window, pieUpdateEvent('spin', '[%d/%d] Downloading' % (curr_pdf_link, num_pdf_links), cite=pdf['pdflink_text'], href=pdf['pdflink']))
            try:
                pdf['outfilename'] = download_pdf(self._notify_window, pdf, author=self._scrapedata['author'], tag=self._scrapedata['tag'])
            except:
                continue
            if not pdf['outfilename']:
                continue # there was a problem, couldn't download?
            int_metadata = get_internal_pdf_metadata(self._notify_window, pdf)
            if int_metadata == None:
                continue # there was a problem, already downloaded?
            pdf.update(int_metadata)
            # by this point, 'pdf' should have the following keys:
            # author, tag, outfilename, pdflink, pdflink_text,
            # pdflink_context, creation_date, download_time,
            # [internal_author, internal_title]
            titles = gen_title_and_docname(pdf, self._scrapecommands['tag_behaviour'])
            # and now - title, final_fn
            pdf['title'] = titles['title']
            try:
                os.renames(pdf['outfilename'], titles['final_fn'])
                pdf['final_fn'] = titles['final_fn']
            except:
                print 'Could not rename file'
                pdf['final_fn'] = pdf['outfilename']
            #reauthor for submission - this begs for a more considered approach;
            #rather dodgy assumption that submissions are always titled 'person's submission...'
            if pdf['pdflink_context'] == 'submissions':
                submatch = re_submission.match(pdf['pdflink_text'])
                if submatch:
                    pdf['author'] = submatch.group(1)
            # user may choose what to in/exclude later
            pdf['bibtex_type'] = config.get('PBoptions', 'default_bibtex_entry_type')
            hpstyle = config.getint('PBoptions', 'default_bibtex_entry_style')
            if hpstyle in (0, 3):
                pdf['howpublished'] = u'On the web'
            elif hpstyle == 1:
                pdf['howpublished'] = u'\\url{%s}' % pdf['pdflink']
            elif hpstyle == 2:
                pdf['howpublished'] = u'\\href{%s}{%s}' % (pdf['pdflink'], 'On the web')
            pdf['url'] = pdf['pdflink']
            pdf['year'] = unicode(time.strftime("%Y", pdf['creation_date']))
            pdf['month'] = unicode(time.strftime("%B", pdf['creation_date']))
            pdf['annote'] = 'Downloaded_on: %s\nPublication_date: %s\nSaved_to: %s\nAuthor_from_metadata: %s\nTitle_from_metadata: %s' % (time.ctime(), time.ctime(time.mktime(pdf['creation_date'])), pdf['final_fn'], pdf['internal_author'], pdf['internal_title'])
            pdf['note'] = ''
            data_id = wx.NewId()
            if pdf['pdflink_context'] in exclude_by_default:
                pdf['exclude'] = True
                wx.PostEvent(self._notify_window, pieUpdateEvent('exclude', 'Success', cite="%s (%s). %s" % (pdf['author'], pdf['year'], pdf['title']), href=pdf['pdflink'], data_id=data_id, updatelast=True))
            else:
                pdf['exclude'] = False
                wx.PostEvent(self._notify_window, pieUpdateEvent('success', 'Success', cite="%s (%s). %s" % (pdf['author'], pdf['year'], pdf['title']), href=pdf['pdflink'], data_id=data_id, updatelast=True))
            return_data[data_id] = pdf
        wx.PostEvent(self._notify_window, ResultEvent(return_data))
 
    def abort(self):
        """abort worker thread."""
        self._want_abort = 1
                         

if __name__ == '__main__':
    print 'ok'
                             



#python

import wx

EVT_PIEUPDATE_ID = wx.NewId()
EVT_RESULT_ID = wx.NewId()
EVT_PIEPREFETCH_ID = wx.NewId()

def EVT_PIEUPDATE(win, func):
    win.Connect(-1, -1, EVT_PIEUPDATE_ID, func)

def EVT_RESULT(win, func):
    """Define Result Event."""
    win.Connect(-1, -1, EVT_RESULT_ID, func)

def EVT_PIEPREFETCH(win, func):
    win.Connect(-1, -1, EVT_PIEPREFETCH_ID, func)
 
    '''Define event updating user (through outputList) of progress of downloads'''
class ResultEvent(wx.PyEvent):
    """Simple event to carry arbitrary result data."""
    def __init__(self, data):
        """Init Result Event."""
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_RESULT_ID)
        self.data = data

class piePrefetchEvent(wx.PyEvent):
    """Simple event to carry prefetched page info."""
    def __init__(self, tag):
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_PIEPREFETCH_ID)
        self.tag = tag

class pieUpdateEvent(wx.PyEvent):
    """Simple event to carry message to the user."""
    def __init__(self, msgtype, msg='', cite=None, href=None, data_id=None, updatelast=False):
        """Init Result Event."""
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_PIEUPDATE_ID)
        self.msgtype = msgtype
        self.msg = msg
        self.cite = cite
        self.href = href
        self.data_id = data_id
        self.updatelast = updatelast



#!/bin/python

import wx
import sys, os, os.path, time, string, re
import ConfigParser
import traceback
import subprocess

from threading import Thread
from pieberry_config import *
from pieberry_settings import *
from pieberry_exec import *
from pieberry_validators import *
from pieberry_events import *
from pieberry_worker import *
from pieberry_output import *
from pieberry_pybtex import *

DEFAULT_SIZE = wx.Size(720, 480) #main window size
TITLE = 'PieBerry Library Assistant'
BUTTON_SIZE = wx.Size(100, 36) #size of main control buttons
smallspace = (10, 10)
tinyspace = (5, 5)

# Button definitions
ID_START = wx.NewId()
ID_STOP = wx.NewId()

class UiTimer(wx.Timer):
    '''timer class that updates outputList's tooltip'''
    def __init__(self, parent, *args, **kwargs):
        wx.Timer.__init__(self, *args, **kwargs)
        self._last_iteration = time.time()
        self._f = parent
 
    def Notify(self):
        self._last_iteration = time.time()
        self._f.onMouseUpdateList()
        self._f.onPrefetchTag(self._last_iteration)

class SpinnyTimer(wx.Timer):
    def __init__(self, parent, *args, **kwargs):
        wx.Timer.__init__(self, *args, **kwargs)
        self._f = parent

    def Notify(self):
        self._f.spinnerTick()
        

class pieMainWindow(wx.Frame):
    '''Main window class for PieBerry'''
    _BUTTON_SIZE = wx.Size(100, 50)
    _DEF_WIDTH = (300, -1)

    def __init__(self, *args, **kwargs):
        wx.Frame.__init__(self, None, -1, TITLE, size=DEFAULT_SIZE, name=TITLE)       
        self._do_layout()
        self.urlField.SetFocus()
        
        # data tracking
        self.allData = {} # store of data for all bibliography items fetched
        self.lastDataKeys = () # tuple of the last batch of bibliography items (keys)
        self.savedDataKeys = [] # list of items in allData that have been saved already (keys)

        # set up tooltips for outputList
        self.mouseOverList = False
        self.outputList.Bind(wx.EVT_ENTER_WINDOW, self.onMouseEnterList)
        self.outputList.Bind(wx.EVT_LEAVE_WINDOW, self.onMouseExitList)
        self.uitimer = UiTimer(self)
        self.uitimer.Start(1000)
        self.spinTimer = None
        self.mouse_last_pos = wx.Point(0,0)
        self.url_text_last_changed = time.time()
        self.tipwindow = None # tooltip for bib data
        self.tipwindow_lastpos = (0,0) # last position tipwindow was created
        self.mouseoveritem = -1 # keep track of where the mouse was hovering
        self.suppress_tipwindow = False # flag to keep the tooltip from popping up in inconvenient places
        # Thread comms
        EVT_RESULT(self, self.onResult)
        EVT_PIEUPDATE(self, self.onUpdate)
        EVT_PIEPREFETCH(self, self.onPrefetchResult)
        self.worker = None

        self.Bind(wx.EVT_CLOSE, self.onClose)

        # # DEBUG - SPOOF DATA
        # wx.PostEvent(self, pieUpdateEvent('success', 'Test item', cite="Test data" , href="Test data", data_id=wx.NewId(), updatelast=False))
        # spoofdata = {
        #     'title': 'Spoof title',
        #     'author': 'Spoof author',
        #     'year': '1833',
        #     'month': 'January',
        #     'howpublished': 'Mungdinugn',
        #     'annote': 'notey',
        #     'download_time': time.localtime(),
        #     'creation_date': time.localtime(),
        #     'final_fn': 'C:noodles',
        #     'pdflink': 'http://www.hell.no',
        #     'url': 'http://www.themong.spo.nn/',
        #     'exclude': True,
        #     'internal_author': 'Jon Hoo',
        #     'internal_title': 'Noh Wai'
        #     }
        # spoof_data_id = wx.NewId()
        # spoofbib = {}
        # spoofbib[spoof_data_id] = spoofdata
        # wx.PostEvent(self, pieUpdateEvent('success', 'Success', cite="%s (%s). %s" % (spoofdata['author'], spoofdata['year'], spoofdata['title']), href=spoofdata['pdflink'], data_id=spoof_data_id, updatelast=False))
        # self.allData.update(spoofbib)
        # self.sc_auth = "Spoof author"
        # self.menu_discard.Enable(True)
        # self.menu_savebibs.Enable(True)
        # # // DEBUG //

    def _do_layout(self):
        _icon = wx.EmptyIcon()
        _icon.CopyFromBitmap(wx.Bitmap(os.path.join(IMGDIR, 'pie_16.png')))
        self.SetIcon(_icon)

        rawlist = os.listdir(config.get('PBoptions', 'workingdir'))
        dirlist = [i for i in rawlist if os.path.isdir(os.path.join(config.get('PBoptions', 'workingdir'), i))]
        dirlist.sort()

        menuBar = wx.MenuBar()
        fileMenu = wx.Menu()
        toolMenu = wx.Menu()
        self.menu_savebibs = wx.MenuItem(fileMenu, -1, '&Save Bibliography Changes [%s]\tCtrl-s' % os.path.basename(config.get('PBoptions', 'default_bibliography')), 'Save')
        self.menu_savebibs.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE, wx.ART_MENU, (16, 16)))
        self.menu_discard = wx.MenuItem(fileMenu, -1, '&Discard Bibliography Changes', 'Discard')
        self.menu_quit = wx.MenuItem(fileMenu, -1, '&Quit\tCtrl-q', 'Quit')
        self.menu_quit.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_QUIT, wx.ART_MENU))
        self.menu_config = wx.MenuItem(fileMenu, -1, 'S&ettings', 'Config')
        self.menu_about = wx.MenuItem(fileMenu, -1, '&About', 'About')
        self.menu_pageref = wx.MenuItem(toolMenu, -1, 'Create &reference for top-level page', 'Reference')
        fileMenu.AppendItem(self.menu_savebibs)
        fileMenu.AppendItem(self.menu_discard)
        fileMenu.AppendItem(self.menu_config)
        fileMenu.AppendItem(self.menu_about)
        fileMenu.AppendItem(self.menu_quit)
        toolMenu.AppendItem(self.menu_pageref)
        # if PYBTEX:
        self.menu_find = wx.MenuItem(toolMenu, -1, '&Find items in bibliography\tCtrl-f', 'Find')
        self.menu_find.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FIND, wx.ART_MENU))
        toolMenu.AppendItem(self.menu_find)
        self.Bind(wx.EVT_MENU, self.onFind, self.menu_find)
        menuBar.Append(fileMenu, '&File')
        menuBar.Append(toolMenu, '&Tools')
        self.SetMenuBar(menuBar)
        self.SetAutoLayout(True)
        self.Bind(wx.EVT_MENU, self.onSaveBibs, self.menu_savebibs)
        self.Bind(wx.EVT_MENU, self.onClose, self.menu_quit)
        self.Bind(wx.EVT_MENU, self.onConfig, self.menu_config)
        self.Bind(wx.EVT_MENU, self.onAbout, self.menu_about)
        self.Bind(wx.EVT_MENU, self.onDiscard, self.menu_discard)
        self.Bind(wx.EVT_MENU, self.onPageReference, self.menu_pageref)
        self.menu_savebibs.Enable(False)
        self.menu_discard.Enable(False)

        topSizer = wx.BoxSizer(wx.VERTICAL)

        panel0 = wx.Panel(self, -1, style=wx.RAISED_BORDER|wx.EXPAND|wx.TAB_TRAVERSAL)
        panel1 = wx.Panel(self, -1, style=wx.NO_BORDER|wx.EXPAND|wx.TAB_TRAVERSAL)
        

        #Various controls
        # ofvalid = pieUrlValidator()
        self.pasteButton = wx.BitmapButton(panel0, -1, wx.ArtProvider.GetBitmap(wx.ART_PASTE, wx.ART_TOOLBAR, (16, 16)), style=wx.NO_BORDER)
        tt7 = wx.ToolTip('Paste from clipboard')
        self.pasteButton.SetToolTip(tt7)
        self.urlField = wx.TextCtrl(panel0, -1, size=self._DEF_WIDTH, validator = pieUrlValidator(), style=wx.TE_PROCESS_ENTER|wx.TE_BESTWRAP)
        self.urlField.ChangeValue('http://')
        self.urlField.SetBackgroundColour("pink")
        tt1 = wx.ToolTip('Website to scan for PDF documents to download.')
        self.urlField.SetToolTip(tt1)
        lburl = wx.StaticText(panel0, -1, 'URL:')

        self.authorField = wx.ComboBox(panel0, -1, size=self._DEF_WIDTH, choices=dirlist, validator=piePlainTextValidator(), style=wx.EXPAND|wx.CB_DROPDOWN)
        tt2 = wx.ToolTip('Generic "author" to use for downloaded documents. This will also be used as a first level subdirectory in which to store these documents.')
        self.authorField.SetToolTip(tt2)

        self.corpAuthorCb = wx.CheckBox(panel0, -1, label="Author is a corporate entity")
        self.corpAuthorCb.SetValue(True)
        tt5 = wx.ToolTip('Treat this author as a corporate entity rather than person(s). This matters to how bibliographies treat these entries.')
        self.corpAuthorCb.SetToolTip(tt5)

        self.tagField = wx.TextCtrl(panel0, -1, size=self._DEF_WIDTH, validator=piePlainTextValidator(), style=wx.EXPAND)
        tt3 = wx.ToolTip('Category tag and second level subdirectory for downloaded documents.')
        self.tagField.SetToolTip(tt3)
        
        pf_use_choices = (
            'Append to title - " ... - Phrase"', 
            'Append to title - " ... (Phrase)"', 
            'Prepend to title - "Phrase: ..."', 
            'Use as subdirectory only')
        self.use_choice_lookup = ('append_hyphen', 'append_brackets', 'prepend', 'dironly')
        self.pfUseChoice = wx.Choice(panel0, -1, choices=pf_use_choices, style=wx.EXPAND)
        tt6 = wx.ToolTip('Select how to use the category phrase. It will always be used to create a subdirectory for the downloaded documents. It may also be appended to the titles of the downloaded documents')
        self.pfUseChoice.SetToolTip(tt6)
        self.pfUseChoice.SetSelection(0)

        self.scanButton = wx.Button(panel0, -1, label='Scan')
        self.outputList = wx.ListCtrl(panel1, -1, style=wx.LC_REPORT|wx.EXPAND)

        self.iList = wx.ImageList(16, 16)
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_blueball16.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_redsquare16.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_yellowex16.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_greensquare16.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_greentick16.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_blueex16.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al00.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al01.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al02.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al03.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al04.png')))
        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al05.png')))

        self.spinnerstate = -1
        self.listcols = ('', 'Document', 'Link')
        self.widthcols = (150, 380, 300)
        self.msgtypes = {
            'spin': 'spin',
            'blank': None,
            'success': 0,
            'fail': 1,
            'warn': 2,
            'pass': 3,
            'tick': 4,
            'exclude': 5
            }
        for i in range(len(self.listcols)):
            self.outputList.InsertColumn(i, self.listcols[i])
            self.outputList.SetColumnWidth(i, self.widthcols[i])
        self.outputList.currentItem = -1
        self.outputList.AssignImageList(self.iList, wx.IMAGE_LIST_SMALL)
        
        sizerA = wx.BoxSizer(wx.HORIZONTAL)

        sizer0 = wx.FlexGridSizer(5, 3, 5, 5)
        sizer0.AddGrowableCol(1, 1)
        sizer0.SetFlexibleDirection(wx.BOTH)
        sizer1 = wx.BoxSizer(wx.VERTICAL)

        sizerA.Add(lburl, 1, wx.EXPAND|wx.ALL, 3)
        sizerA.Add(self.pasteButton, 0, wx.ALL, 3)
        sizer0.Add(sizerA, 1, wx.EXPAND)
        sizer0.Add(self.urlField, 1, wx.EXPAND|wx.ALL, 3)
        sizer0.Add(self.scanButton, 1, wx.EXPAND|wx.ALL, 3)
        sizer0.Add(wx.StaticText(panel0, -1, "Default author:"), 1, wx.EXPAND|wx.ALL, 3)
        sizer0.Add(self.authorField, 1, wx.EXPAND|wx.ALL, 3)
        sizer0.Add(self.corpAuthorCb, 1, wx.EXPAND|wx.ALL, 3)
        sizer0.Add(wx.StaticText(panel0, -1, 'Category phrase:'), 1, wx.EXPAND|wx.ALL, 3)
        sizer0.Add(self.tagField, 1, wx.EXPAND|wx.ALL, 3)
        sizer0.Add(self.pfUseChoice, 1, wx.EXPAND|wx.ALL, 3)

        # sizer1.Add(self.outputField, 1, wx.EXPAND)
        sizer1.Add(self.outputList, 2, wx.EXPAND)
        panel0.SetSizer(sizer0)
        panel1.SetSizer(sizer1)
        topSizer.Add(panel0, 0, wx.EXPAND)
        topSizer.Add(panel1, 1, wx.EXPAND)
        self.SetSizer(topSizer)

        self.Layout()

        wx.EVT_LIST_ITEM_RIGHT_CLICK(self.outputList, -1, self.onListRightClick)
        self.list_item_clicked = None
        self.urlField.Bind(wx.EVT_TEXT_ENTER, self.onScan)
        wx.EVT_LIST_ITEM_ACTIVATED(self.outputList, -1, self.onListDoubleClick)
        self.authorField.Bind(wx.EVT_TEXT, self.onAuthorValidate)
        self.tagField.Bind(wx.EVT_TEXT, self.onTagValidate)
        self.scanButton.Bind(wx.EVT_BUTTON, self.onScan)
        # self.urlField.Bind(wx.EVT_KILL_FOCUS, self.onUrlTextChanged)
        self.urlField.Bind(wx.EVT_TEXT, self.onUrlTextChanged)
        self.pasteButton.Bind(wx.EVT_BUTTON, self.onPasteButton)

    def _do_repop_authlist(self):
        #list of candidates for authors - take from subdir names in data directory
        rawlist = os.listdir(config.get('PBoptions', 'workingdir'))
        dirlist = [i for i in rawlist if os.path.isdir(os.path.join(config.get('PBoptions', 'workingdir'), i))]
        dirlist.sort()
        dirlist.reverse()
        self.authorField.Clear()
        for rd in dirlist:
            self.authorField.Insert(rd, 0)

    def outputMsg(self, idx, msgtype, msg='', cite=None, href=None, data_id=None, updatelast=False):
        '''posts a message with appropriate icon to the display 
        types - blank, success, fail, warn, pass'''
        if updatelast: 
            self.outputList.DeleteItem(self.outputList.currentItem)
            thisidx = self.outputList.currentItem
        else: 
            thisidx = idx
        if msgtype == 'blank':
            if self.spinTimer: self.spinnerStop()
            nexidx = self.outputList.InsertStringItem(thisidx, msg)
        else:
            if msgtype == 'spin':
                nexidx = self.outputList.InsertImageStringItem(thisidx, msg, 6)
                self.spinnerStart(nexidx)
            else:
                if self.spinTimer: self.spinnerStop()
                nexidx = self.outputList.InsertImageStringItem(thisidx, msg, self.msgtypes[msgtype])
        # set reference to store of bibliography data against the item
        if not data_id is None:
            self.outputList.SetItemData(nexidx, data_id)
        if cite:
            self.outputList.SetStringItem(nexidx, 1, cite)
        if href:
            self.outputList.SetStringItem(nexidx, 2, href)
        self.outputList.currentItem = nexidx
        self.outputList.EnsureVisible(nexidx) # scroll to here
        return nexidx

    def onFind(self, evt):
        self.suppress_tipwindow = True
        if tuple(self.allData.keys()) != tuple(self.savedDataKeys): #check whether we've saved all this to the .bib file before exiting
            dia = wx.MessageDialog(self, 'You have unsaved bibliography data for downloaded files.\nSave these to your master bibliography file before exiting?', 'Unsaved items', style=wx.YES|wx.NO|wx.CANCEL)
            ans = dia.ShowModal()
            if ans == wx.ID_CANCEL: return
            if ans == wx.ID_YES: self.onSaveBibs()
        tedia = wx.TextEntryDialog(self, 'Find items (Syntax: "field1==term1;term2, field1==term3 ...)', 'Find items in bibliography matching:', defaultValue='author==')
        tedo = tedia.ShowModal()
        self.suppress_tipwindow = False
        if tedo == wx.ID_CANCEL:
            return
        searchphrase = tedia.GetValue()
        if len(searchphrase.strip()) == 0:
            return
        self.outputList.DeleteAllItems()
        self.allData = {}
        self.savedDataKeys = []
        # from pieberry_pybtex import get_top_search_results
        res = get_top_search_results(searchphrase)
        for bibdata in res:
            newid = wx.NewId()
            print newid
            self.allData[newid] = bibdata
            wx.PostEvent(
                self, 
                pieUpdateEvent(
                    'success', 
                    'Found', 
                    cite="%s (%s). %s" % (bibdata['author'], bibdata['year'], bibdata['title']), 
                    href=bibdata['pdflink'], 
                    data_id=newid, 
                    updatelast=False))
        self.savedDataKeys = self.allData.keys()
        self.menu_savebibs.Enable(False)
        self.menu_discard.Enable(False)

    def onPasteButton(self, evt):
        if not wx.TheClipboard.IsOpened():
            wx.TheClipboard.Open()
            do = wx.TextDataObject()
            success = wx.TheClipboard.GetData(do)
            wx.TheClipboard.Close()
            if success:
                self.urlField.SetValue(do.GetText())

    def onAbout(self, evt):
        info = wx.AboutDialogInfo()
        info.AddDeveloper('Raif Sarcich')
        info.SetVersion('1.6-1')
        info.SetCopyright('(c) 2010 Raif Sarcich et. al.')
        info.SetLicence('''This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.''')
        info.SetDescription('A program to download pdf documents from public websites, and catalogue them in BibTeX format')
        info.SetName('Pieberry (...for your library)')
        _icon = wx.EmptyIcon()
        _icon.CopyFromBitmap(wx.Bitmap(os.path.join(IMGDIR, 'pie_48.png')))
        info.SetIcon(_icon)
        wx.AboutBox(info)

    def onMouseEnterList(self, evt):
        self.mouseOverList = True

    def onMouseExitList(self, evt):
        self.mouseOverList = False
        if self.tipwindow and sys.platform == 'linux2': #irritating non x-platform behaviour 
            self.tipwindow.Destroy()
            self.tipwindow = None

    def onPrefetchTag(self, last_iter):
        if 2 > (last_iter - self.url_text_last_changed) > 1:
            if self.urlField.GetValidator().Validate():
                prefetcher = piePrefetchThread(self, self.urlField.GetValue())
            print 'trigger fetch?'

    def onPrefetchResult(self, evt):
        self.tagField.ChangeValue(evt.tag)
        
    def onMouseUpdateList(self):
        '''work out where the mouse is in the outputList, update
        tooltip accordingly'''
        # three-way switch....
        xy = wx.GetMousePosition()
        mlp = self.mouse_last_pos
        self.mouse_last_pos = xy
        # print xy, mlp, mlp == xy
        # do nothing where we're suppressing tip window
        if self.suppress_tipwindow:
            return
        # do nothing if the mouse isn't over the list
        if not self.mouseOverList:
            return
        ol_xy = self.outputList.ScreenToClient(xy)
        it_idx, flags = self.outputList.HitTest(ol_xy)
        # if there's nothing there, kill any windows
        if it_idx == wx.NOT_FOUND: 
            if self.tipwindow: 
                self.tipwindow.Destroy()
                self.tipwindow = None
            self.mouseoveritem = it_idx
            return
        # if the mouse is static but there's already a tipwindow, keep it
        if mlp == xy and self.tipwindow:
            return
        # if the mouse is moving and there's a tipwindow, kill it
        if mlp != xy and self.tipwindow:
            self.tipwindow.Destroy()
            self.tipwindow = None
            self.mouseoveritem = it_idx
            return
        # if the mouse is static, and there's no tipwindow, create one
        if mlp == xy and not self.tipwindow: 
            self.mouseoveritem = it_idx
            bibidx = self.outputList.GetItemData(it_idx)
            if self.allData.has_key(bibidx) and not (ol_xy == self.tipwindow_lastpos): # if there's data to display...
                bibdata = self.allData[bibidx]
                ttdata = "Author: %s\nTitle: %s\nPublication Date: %s %s\nHow published: %s" % (bibdata['author'], bibdata['title'], bibdata['month'], bibdata['year'], bibdata['howpublished'][:50])
                self.tipwindow = wx.TipWindow(self.outputList, ttdata, maxLength=300)
                self.tipwindow_lastpos = ol_xy
                if sys.platform == 'win32': # tipwindow behaviour is annoyingly non x-platform
                    wx.FutureCall(2000, self.tipwindow.Close)
                    return
            else: # if there's no data associated with this item, do nothing
                if self.tipwindow: self.tipwindow.Destroy()
                self.tipwindow = None
                return

    def onListRightClick(self, evt):
        '''pop up appropriate context menu'''
        self.suppress_tipwindow = True # we don't want tips while we're menu-ing
        if self.tipwindow:
            self.tipwindow.Destroy()
            self.tipwindow = None
        # record what was clicked
        if evt.GetIndex() == -1: return
        self.list_item_clicked = right_click_context = evt.GetIndex()
        ### 2. Launcher creates wxMenu. ###
        menu = wx.Menu()
        copyMenu = wx.Menu()
        it_idx = self.outputList.GetItemData(self.list_item_clicked)
        if self.allData.has_key(it_idx):
            rcm_opendoc = wx.MenuItem(menu, 0, 'Open document')
            rcm_opendoc.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_MENU))
            menu.AppendItem(rcm_opendoc)
            wx.EVT_MENU(menu, 0, self.onMenuOpenDocument)
            rcm_openfol = wx.MenuItem(menu, 1, 'Open document\'s folder')
            rcm_openfol.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FOLDER_OPEN, wx.ART_MENU))
            menu.AppendItem(rcm_openfol)
            wx.EVT_MENU(menu, 1, self.onMenuOpenDocumentFolder)
            rcm_deletedoc = wx.MenuItem(menu, 2, 'Delete document from disk')
            rcm_deletedoc.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_DELETE, wx.ART_MENU))
            menu.AppendItem(rcm_deletedoc)
            wx.EVT_MENU(menu, 2, self.onDelete)
            # special functionality from pybtex
            if self.allData[it_idx].has_key('bibtex_key'):
                rcm_copybibkey = wx.MenuItem(copyMenu, 8, 'Copy BibTeX key to clipboard')
                rcm_copybibkey.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
                copyMenu.AppendItem(rcm_copybibkey)
                self.Bind(wx.EVT_MENU, self.onCopyBibtexKey, rcm_copybibkey)
        # if PYBTEX:
            rcm_copy_citation = wx.MenuItem(copyMenu, 6, 'Copy citation to clipboard (html)')
            rcm_copy_citation.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
            copyMenu.AppendItem(rcm_copy_citation)
            self.Bind(wx.EVT_MENU, self.onCopyCitation, rcm_copy_citation)
            rcm_copy_citation_plain = wx.MenuItem(copyMenu, 9, 'Copy citation to clipboard (text)')
            rcm_copy_citation_plain.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
            copyMenu.AppendItem(rcm_copy_citation_plain)
            self.Bind(wx.EVT_MENU, self.onCopyCitationPlain, rcm_copy_citation_plain)

        if self.allData.has_key(it_idx) and not it_idx in self.savedDataKeys:
            menu.Append(3, 'Edit bibliography entry')
            wx.EVT_MENU(menu, 3, self.onMenuEditBibEntry)
            rcm_exclude = wx.MenuItem(menu, 4, 'Exclude from bibliography', kind=wx.ITEM_CHECK)
            menu.AppendItem(rcm_exclude)
            if self.allData[it_idx]['exclude'] == True:
                menu.Check(4, True)
            wx.EVT_MENU(menu, 4, self.onMenuToggleExclude)
        item = self.outputList.GetItem(self.list_item_clicked, 2)
        if len(item.GetText()) > 0:
            rcm_copyurl = wx.MenuItem(copyMenu, 5, 'Copy Web Url to Clipboard')
            rcm_copyurl.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
            copyMenu.AppendItem(rcm_copyurl)
            self.Bind(wx.EVT_MENU, self.onMenuCopyUrl, rcm_copyurl)
        menu.AppendMenu(9, 'Copy ...', copyMenu)
        # menu.Append(5, 'Debug: item data')
        # wx.EVT_MENU(menu, 5, self.onMenuDebugData)
        ### 5. Launcher displays menu with call to PopupMenu, invoked on the source component, passing event's GetPoint. ###
        self.outputList.PopupMenu( menu, evt.GetPoint() )
        menu.Destroy() # destroy to avoid mem leak
        self.suppress_tipwindow = False

    def onMenuToggleExclude(self, evt):
        data_id = self.outputList.GetItemData(self.list_item_clicked)
        if self.allData[data_id]['exclude'] == True:
            self.allData[data_id]['exclude'] = False
            self.outputList.SetItemImage(self.list_item_clicked, self.msgtypes['success'])
        else:
            self.allData[data_id]['exclude'] = True
            self.outputList.SetItemImage(self.list_item_clicked, self.msgtypes['exclude'])

    def onCopyBibtexKey(self, evt):
        data_id = self.outputList.GetItemData(self.list_item_clicked)
        if self.allData[data_id].has_key('bibtex_key'):
            clipdata = wx.TextDataObject()
            clipdata.SetText(self.allData[data_id]['bibtex_key'])
            wx.TheClipboard.Open()
            wx.TheClipboard.SetData(clipdata)
            wx.TheClipboard.Close()
        
    def onCopyCitationPlain(self, evt):
        data_id = self.outputList.GetItemData(self.list_item_clicked)
        pybobj = pieberry_to_pybtex(self.allData[data_id])
        citetext = get_formatted_citation(pybobj, format='plaintext')
        clipdata = wx.TextDataObject()
        clipdata.SetText(citetext)
        wx.TheClipboard.Open()
        wx.TheClipboard.SetData(clipdata)
        wx.TheClipboard.Close()
        if PYNOTIFY:
            import pynotify
            n = pynotify.Notification("Citation", citetext, os.path.join(IMGDIR, 'pie_48.png'))
            n.show()

    def onCopyCitation(self, evt):
        data_id = self.outputList.GetItemData(self.list_item_clicked)
        # from pieberry_pybtex import get_formatted_citation_html
        pybobj = pieberry_to_pybtex(self.allData[data_id])
        if sys.platform == 'win32':
            from pieberry_htmldataobject import HTMLDataObject
            citetext = get_formatted_citation(pybobj, format='html')
            clipdata = HTMLDataObject()
            clipdata.SetValue(citetext)
        else:
            citetext = get_formatted_citation(pybobj, format='html')
            clipdata = wx.TextDataObject()
            clipdata.SetText(citetext)
            fmt = wx.DataFormat(wx.DF_INVALID)
            fmt.SetId('text/html')
            clipdata.SetFormat(fmt)

        wx.TheClipboard.Open()
        wx.TheClipboard.SetData(clipdata)
        wx.TheClipboard.Close()
        if PYNOTIFY:
            import pynotify
            n = pynotify.Notification("Citation", get_formatted_citation(pybobj, format='plaintext'), os.path.join(IMGDIR, 'pie_48.png'))
            n.show()

    def onListDoubleClick(self, evt):
        if evt.GetIndex() == -1: return
        self.list_item_clicked = evt.GetIndex()
        self.onMenuOpenDocument(1)

    def onMenuDebugData(self, evt):
        data_id = self.outputList.GetItemData(self.list_item_clicked)
        print 'list_item_clicked = %d' % self.list_item_clicked
        print 'list item data = %d' % data_id
        if self.allData.has_key(data_id):
            print self.allData[data_id]
        else:
            print 'No data associated with this item'

    def onMenuOpenDocument(self, evt):
        data_id = self.outputList.GetItemData(self.list_item_clicked)
        if self.allData.has_key(data_id):
            if sys.platform == 'linux2':
                subprocess.call(('xdg-open', self.allData[data_id]['final_fn']))
            elif sys.platform == 'win32':
                os.startfile(self.allData[data_id]['final_fn'])

    def onMenuOpenDocumentFolder(self, evt):
        data_id = self.outputList.GetItemData(self.list_item_clicked)
        if sys.platform == 'linux2':
            subprocess.call(('xdg-open', os.path.dirname(self.allData[data_id]['final_fn'])))
        elif sys.platform == 'win32':
            os.startfile(os.path.dirname(self.allData[data_id]['final_fn']))
        elif sys.platform == 'darwin':
            subprocess.call(('open', os.path.dirname(self.allData[data_id]['final_fn'])))

    def onMenuCopyUrl(self, evt):
        '''copy the downloaded file's URL to the clipboard'''
        item = self.outputList.GetItem(self.list_item_clicked, 2)
        clipdata = wx.URLDataObject()
        clipdata.SetURL(item.GetText())
        wx.TheClipboard.Open()
        wx.TheClipboard.SetData(clipdata)
        wx.TheClipboard.Close()

    def onMenuEditBibEntry(self, evt):
        idx = self.outputList.GetItemData(self.list_item_clicked)
        if idx in self.savedDataKeys:
            wx.MessageBox("This has already been saved.\nYou can use a bibliography manager like JabRef\n to maintain your library.", "Error")
            return
        if self.allData.has_key(idx):
            self.suppress_tipwindow = True
            dia = pieNewBibEditDialog(self.allData[idx], self)
            res = dia.ShowModal()
            if res == wx.ID_OK:
            #TODO data integrity/validity checking
                self.allData[idx] = dia.getData()
                self.outputList.SetItemText(self.list_item_clicked, 'Updated')
                self.outputList.SetItemImage(self.list_item_clicked, self.msgtypes['pass'])
                self.outputList.SetStringItem(self.list_item_clicked, 1, "%s (%s). %s" % (self.allData[idx]['author'].strip('{} '), self.allData[idx]['year'], self.allData[idx]['title']))
            dia.Destroy()
            self.suppress_tipwindow = False
        else: print 'Tried to edit non-entry entry'

    def onAuthorValidate(self, evt=1):
        self.authorField.GetValidator().Validate()
        self.authorField.Refresh()

    def onTagValidate(self, evt=1):
        self.tagField.GetValidator().Validate()
        self.tagField.Refresh()

    def onUrlTextChanged(self, evt=1):
        self.url_text_last_changed = time.time()
        self.urlField.GetValidator().Validate()
        self.urlField.Refresh() 

    def onClose(self, evt=1):
        '''called when top level window closes'''
        print tuple(self.allData.keys())
        print tuple(self.savedDataKeys)
        if tuple(self.allData.keys()) != tuple(self.savedDataKeys): #check whether we've saved all this to the .bib file before exiting
            dia = wx.MessageDialog(self, 'You have unsaved bibliography data for downloaded files.\nSave these to your master bibliography file before exiting?', 'Unsaved items', style=wx.YES|wx.NO|wx.CANCEL)
            ans = dia.ShowModal()
            if ans == wx.ID_CANCEL: return
            if ans == wx.ID_YES: self.onSaveBibs()
        self.Destroy()

    def onScan(self, evt=1):
        self.sc_url = self.urlField.GetValue().strip()
        self.sc_tag = self.tagField.GetValue().strip()
        self.sc_auth = self.authorField.GetValue().strip()
        self.sc_tagbehav = self.use_choice_lookup[self.pfUseChoice.GetCurrentSelection()]
        self.sc_corpauth = self.corpAuthorCb.IsChecked()
        if not (self.urlField.GetValidator().Validate() and self.authorField.GetValidator().Validate() and self.tagField.GetValidator().Validate()):
            self.outputMsg(sys.maxint, 'warn', 'Invalid Input')
            return
        scrapedata = {
            'url': self.sc_url, 
            'tag': self.sc_tag, 
            'author': self.sc_auth
            }
        scrapecommands = {
            'tag_behaviour': self.sc_tagbehav
            }
        if not self.worker:
            self.scanButton.Disable()
            self.worker = pieWorkerThread(self, scrapedata, scrapecommands)

    def onPageReference(self, evt):
        '''Create a reference for the url itself, rather than pdfs it contains'''
        if self.urlField.GetValidator().Validate():
            bibdata = {}
            bibdata['author'] = self.authorField.GetValue()
            bibdata['title'] = self.tagField.GetValue()
            bibdata['url'] = self.urlField.GetValue()
            bibdata['howpublished'] = 'On the web'
            bibdata['year'] = unicode(time.strftime("%Y"))
            bibdata['month'] = unicode(time.strftime("%B"))
            bibdata['creation_date'] = time.localtime()
            bibdata['download_time'] = time.localtime()
            bibdata['final_fn'] = ''
            bibdata['annote'] = bibdata['note'] = ''
            bibdata['exclude'] = False
            bibdata['internal_author'] = ''
            bibdata['internal_title'] = ''
            data_idx = wx.NewId()
            self.allData[data_idx] = bibdata
            wx.PostEvent(self, pieUpdateEvent('success', 'Success', cite="%s (%s). %s" % (bibdata['author'], bibdata['year'], bibdata['title']), href=bibdata['url'], data_id=data_idx, updatelast=False))
            self.menu_savebibs.Enable(True)
            self.menu_discard.Enable(True)

    def onStop(self, evt):
        if self.worker:
            print "Aborting..."
            self.worker.abort()

    def onSaveBibs(self, evt=1):
        '''Save main bibliography (post-editing)''' 
        print 'savebibs'
        savedata = ''
        saved_item_keys = []
        try:
            bibwriter = pieAllWriter()
            for key, value in self.allData.items():
                if not key in self.savedDataKeys:
                    # ensure author is treated as correct type (corp/person)
                    bibentry = self.allData[key]
                    if not self.allData[key].has_key('corporate_author'):
                        if bibentry['author'] == self.sc_auth:
                            bibentry['corporate_author'] = self.corpAuthorCb.IsChecked()
                        else:
                            bibentry['corporate_author'] = False
                    bibwriter.addEntry(bibentry)
                    # print 'adding entry %s' % bibentry['title']
                    saved_item_keys.append(key)
            bibwriter.write()

            # check off saved items in the list view
            for i in range(self.outputList.GetItemCount()):
                idat = self.outputList.GetItemData(i)
                if idat in saved_item_keys: 
                    if self.allData[idat]['exclude'] == False:
                        self.outputList.SetItemImage(i, self.msgtypes['tick'])
                        self.outputList.SetItemText(i, 'Added to bibliography')
                    else:
                        self.outputList.SetItemText(i, 'Not added to bibliography')
                        
            self.savedDataKeys = self.allData.keys()
            self.menu_savebibs.Enable(False)
            self.menu_discard.Enable(False)
            self.SetTitle(TITLE)
        except Exception:
            traceback.print_exc()
            wx.MessageBox(traceback.format_exc())

    def onDiscard(self, evt=1):
        '''discard data intended for the bibliography'''
        dia = wx.MessageDialog(self, "Do you want to jettison the bibliography information Pieberry has collected?", "Discard bibliography information", style=wx.YES_NO|wx.ICON_QUESTION)
        ans = dia.ShowModal()
        if ans == wx.ID_YES:
            nonsaved_item_keys = []
            for key, value in self.allData.items():
                if not key in self.savedDataKeys:
                    nonsaved_item_keys.append(key)
            for i in range(self.outputList.GetItemCount()):
                idat = self.outputList.GetItemData(i)
                if idat in nonsaved_item_keys:
                    self.outputList.SetItemText(i, 'Not added to bibliography')
            self.savedDataKeys = self.allData.keys()
            self.menu_savebibs.Enable(False)
            self.menu_discard.Enable(False)
            self.SetTitle(TITLE)

    def onDelete(self, evt):
        idx = self.outputList.GetItemData(self.list_item_clicked)
        delfileinfo = self.allData[idx]
        dia = wx.MessageDialog(self, "Delete file %s" % delfileinfo['final_fn'], "Delete file", style=wx.YES_NO|wx.ICON_EXCLAMATION)
        ans = dia.ShowModal()
        if ans == wx.ID_YES:
            try:
                os.remove(delfileinfo['final_fn'])
                self.outputList.SetItemImage(self.list_item_clicked, self.msgtypes['fail'])
                self.outputList.SetItemText(self.list_item_clicked, 'Deleted')
                self.allData.pop(idx)
            except:
                traceback.print_exc()
                wx.MessageBox('Could not delete file', 'Error')

    def onResult(self, evt):
        if evt.data is None:
            print "Aborted!"
            self.outputMsg(sys.maxint, 'blank', 'Aborted')
        else:
            print 'Downloaded!'
            self.outputMsg(sys.maxint, 'blank', 'Done')
            self.allData.update(evt.data)
            self.lastDataKeys = evt.data.keys()
            savedata = ''
            if len(self.lastDataKeys) > 0: # don't bother if there's nothing new
                writer = pieBibtexWriter()
                for idx in self.lastDataKeys:
                    bibentry = self.allData[idx]
                    # if bibentry['author'] == self.sc_auth:
                    #     bibentry['corporate_author'] = self.corpAuthorCb.IsChecked()
                    writer.addEntry(bibentry)
                writer.setLocation(os.path.join(config.get('PBoptions', 'workingdir'), self.sc_auth, self.sc_tag, '%s_bibliography.bib' % time.strftime('%Y%m%d-%H%M')))
                writer.write()
                self.menu_savebibs.Enable()
                self.menu_discard.Enable()
                self.SetTitle('%s (unsaved)' % TITLE)
        self.worker = None
        self.scanButton.Enable()

    def onUpdate(self, evt):
        print evt.msg
        self.outputMsg(sys.maxint, evt.msgtype, evt.msg, evt.cite, evt.href, evt.data_id, evt.updatelast)

    def onConfig(self, evt):
        self.suppress_tipwindow = True # tip windows get in the way
        cfd = pieNewConfigDialog(config, inipath, self)
        a = cfd.ShowModal()
        if a == wx.ID_OK:
            self._do_repop_authlist()
            self.menu_savebibs.SetItemLabel('&Save Bibliography Data [%s]\tCtrl-s' % os.path.basename(config.get('PBoptions', 'default_bibliography')))
        self.suppress_tipwindow = False

    def spinnerStart(self, list_item):
        '''start a spinning icon to indicate download'''
        self.spin_list_item = list_item
        self.spinTimer = SpinnyTimer(self)
        self.spinTimer.Start(200)

    def spinnerTick(self):
        '''progress through the icon animation'''
        if self.spinnerstate == 5:
            self.spinnerstate = -1
        self.spinnerstate += 1
        sli = self.spinnerstate + 6
        self.outputList.SetItemImage(self.spin_list_item, sli)
        
    def spinnerStop(self):
        self.spinTimer.Stop()
        self.spinTimer.Destroy()
        self.spinTimer = None

class pieNewConfigDialog(pieConfigDialog):
    def __init__(self, config, configfile, *args, **kwargs):
        pieConfigDialog.__init__(self, *args, **kwargs)
        self._config = config
        self._configfile = configfile
        self.workDirCtrl.SetValue(self._config.get('PBoptions', 'workingdir'))
        self.mainBibCtrl.SetValue(self._config.get('PBoptions', 'default_bibliography'))
        self.tsvBibCb.SetValue(self._config.getboolean('PBoptions', 'export_tsv'))
        self.defaultEntryTypeChoice.SetSelection(self._config.getint('PBoptions', 'default_bibtex_entry_style'))

    def onOk(self, evt):
        self._config.set('PBoptions', 'workingdir', self.workDirCtrl.GetValue())
        self._config.set('PBoptions', 'default_bibliography', self.mainBibCtrl.GetValue())
        self._config.set('PBoptions', 'default_bibtex_entry_style', self.defaultEntryTypeChoice.GetSelection())
        self._config.set('PBoptions', 'default_bibtex_entry_type', ENTRY_TYPE_KEYS[self.defaultEntryTypeChoice.GetSelection()])
        self._config.set('PBoptions', 'export_tsv', unicode(self.tsvBibCb.GetValue()))
        self._config.write(open(self._configfile, 'w'))
        self.EndModal(wx.ID_OK)

    def onCancel(self, evt):
        self.EndModal(wx.ID_CANCEL)

    def onSelectRoot(self, evt):
        fd = wx.DirDialog(self, 'Choose root directory for library', os.getcwd(), style=wx.DD_DIR_MUST_EXIST)
        if fd.ShowModal() == wx.ID_OK:
            newdir = fd.GetPath()
            self.workDirCtrl.SetValue(newdir)
        fd.Close()
        
    def onSelectBib(self, evt):
        dn = os.path.dirname(self.mainBibCtrl.GetValue())
        fd = wx.FileDialog(self, 'Choose principal bibliography file', dn, wildcard="*.bib")
        if fd.ShowModal() == wx.ID_OK:
            newbib = fd.GetPath()
            self.mainBibCtrl.SetValue(newbib)
        fd.Close()
        
class pieNewBibEditDialog(pieBibEditDialog):
    def __init__(self, bibdata, *args, **kwargs):
        pieBibEditDialog.__init__(self, *args, **kwargs)
        self._bibdata = bibdata
        self.keyAutoCb.SetValue(True)
        self.keyCtrl.SetValue(autogen_bibtex_key(bibdata))
        self.keyCtrl.Enable(False)
        self.keyCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
        self.keyAutoCb.Bind(wx.EVT_CHECKBOX, self.onKeygenToggle)
        if not bibdata.has_key('corporate_author'):
            self.authorIsCorporateCb.Set3StateValue(wx.CHK_UNDETERMINED)
        elif bibdata['corporate_author'] == True:
            self.authorIsCorporateCb.Set3StateValue(wx.CHK_CHECKED)
        else:
            self.authorIsCorporateCb.Set3StateValue(wx.CHK_UNCHECKED)
        self.authorCtrl.SetValue(bibdata['author'])
        self.authorCtrl.Bind(wx.EVT_TEXT_ENTER, self.onOk)
        self.authorCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
        self.authorSwapButton.Bind(wx.EVT_BUTTON, self.onAuthorSwap)
        self.authorAltCtrl.SetValue(bibdata['internal_author'])
        self.titleCtrl.SetValue(bibdata['title'])
        self.titleCtrl.Bind(wx.EVT_TEXT_ENTER, self.onOk)
        self.titleCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
        self.titleSwapButton.Bind(wx.EVT_BUTTON, self.onTitleSwap)
        self.titleAltCtrl.SetValue(bibdata['internal_title'])
        self.howpublishedCtrl.SetValue(bibdata['howpublished'])
        self.howpublishedCtrl.Bind(wx.EVT_TEXT_ENTER, self.onOk)
        self.howpublishedCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
        self.noteCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
        self.annoteCtrl.SetValue(bibdata['annote'])
        self.annoteCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
        t = wx.DateTimeFromTimeT(time.mktime(bibdata['creation_date']))
        self.datePicker.SetValue(t)
        self.authorCtrl.SetFocus()

    def onKeygenToggle(self, evt):
        if self.keyAutoCb.IsChecked():
            self.keyCtrl.Enable(False)
        else:
            self.keyCtrl.Enable(True)
        
    def onOk(self, evt):
        for ctrl in (self.keyCtrl, self.authorCtrl, self.titleCtrl, self.howpublishedCtrl, self.annoteCtrl):
            if not ctrl.GetValidator().Validate():
                wx.MessageBox("One or more of these fields has problematic syntax for BibTeX - please remedy")
                return
        self._bibdata['author'] = self.authorCtrl.GetValue()
        self._bibdata['title'] = self.titleCtrl.GetValue()
        self._bibdata['howpublished'] = self.howpublishedCtrl.GetValue()
        self._bibdata['annote'] = self.annoteCtrl.GetValue()
        self._bibdata['note'] = self.noteCtrl.GetValue()
        newdate = wxdate2pydate(self.datePicker.GetValue())
        self._bibdata['year'] = unicode(newdate.year)
        self._bibdata['creation_date'] = newdate.timetuple()
        self._bibdata['month'] = time.strftime('%B', self._bibdata['creation_date'])
        if not self.keyAutoCb.IsChecked():
            self._bibdata['bibtex_key'] = self.keyCtrl.GetValue()
        corpcb = self.authorIsCorporateCb.Get3StateValue()
        if corpcb == wx.CHK_UNCHECKED:
            self._bibdata['corporate_author'] = False
        elif corpcb == wx.CHK_CHECKED:
            self._bibdata['corporate_author'] = True
        self.EndModal(wx.ID_OK)

    def onCancel(self, evt):
        self.EndModal(wx.ID_CANCEL)

    def getData(self):
        return self._bibdata
    
    def onTitleSwap(self, evt):
        t = self.titleCtrl.GetValue()
        self.titleCtrl.SetValue(self.titleAltCtrl.GetValue())
        self.titleAltCtrl.SetValue(t)

    def onAuthorSwap(self, evt):
        t = self.authorCtrl.GetValue()
        self.authorCtrl.SetValue(self.authorAltCtrl.GetValue())
        self.authorAltCtrl.SetValue(t)

    def onFieldValidate(self, evt):
        obj = evt.GetEventObject()
        obj.GetValidator().Validate()

# class RedirectText(object):
#     def __init__(self,aWxTextCtrl):
#         self.out=aWxTextCtrl
#     def write(self,string):
#         self.out.WriteText(string)

def main(argv):
    sys.stdout = open(os.path.join(sysdir, 'stdlog.txt'), 'w')
    sys.stderr = open(os.path.join(sysdir, 'errlog.txt'), 'w')
    app = wx.App(redirect=False)
    w = pieMainWindow()
    w.Show()
    # handle urls passed to pieberry from the command line
    if len(argv) > 1:
        regexp_long = re.compile("([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}|(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+)(:[0-9]*)?/*[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\),\\\"]")
        if regexp_long.match(argv[1]):
            w.urlField.SetValue(argv[1])
        elif argv[1] in ('-h', '--help', '/?', '/h'):
            usage()
            sys.exit(0)
        else:
            usage()
            dia = wx.MessageDialog(w, 'Argument passed to Pieberry was not a valid URL - Exiting', 'Error', style=wx.OK)
            dia.ShowModal()
            sys.exit(2)

    app.MainLoop()

def usage():
    print """usage: pieberrydm [URL]"""

if __name__ == '__main__':
    main(sys.argv)



#! python
import ConfigParser
import sys, os, string

def setup_config(inipath):
    config = ConfigParser.RawConfigParser()
    config.read(inipath)
    if not config.has_section('PBoptions'):
        config.add_section('PBoptions')
    if not config.has_option('PBoptions', 'workingdir'):
        config.set('PBoptions', 'workingdir', os.getcwd())
    if not config.has_option('PBoptions', 'default_bibliography'):
        config.set('PBoptions', 'default_bibliography', os.path.join(sysdir, 'library.bib'))
    if not config.has_option('PBoptions', 'exclude_by_default'):
        config.set('PBoptions', 'exclude_by_default', 'submissions')
    if not config.has_option('PBoptions', 'default_bibtex_entry_type'):
        config.set('PBoptions', 'default_bibtex_entry_type', 'Misc')
    if not config.has_option('PBoptions', 'default_bibtex_entry_style'):
        config.set('PBoptions', 'default_bibtex_entry_style', '0')
    if not config.has_option('PBoptions', 'default_bibtex_url_handling'):
        config.set('PBoptions', 'default_bibtex_url_handling', 'href_howpublished')
    if not config.has_option('PBoptions', 'export_tsv'):
        config.set('PBoptions', 'export_tsv', 'false')
    config.write(open(inipath, 'w'))
    return config

def GetAppdir():
    '''get the directory of the application itself'''
    if not __file__:
        pathname = sys.argv[0]
        retval = os.path.split(os.path.abspath(pathname))[0]
    else:
        retval = os.path.dirname(__file__)
    return retval

print 'appdir - ', GetAppdir()
APPNAME = "pieberry"
if sys.platform == 'linux2':
    sysdir = os.path.join(os.environ["HOME"], "".join([".", APPNAME]))
elif sys.platform == 'win32':
    if os.path.exists(os.path.join(os.environ["HOMEPATH"], "Application Data")):
        sysdir = os.path.join(os.environ["HOMEPATH"], "Application Data", APPNAME)
    else:
        sysdir = os.getcwd()
else: 
    sysdir = os.getcwd()
if not os.path.exists(sysdir):
    os.mkdir(sysdir)
inipath = os.path.join(sysdir, 'pieberry.ini')
config = setup_config(inipath)
DEFAULT_BIBLIOGRAPHY = config.get('PBoptions', 'default_bibliography')
if not (os.path.exists(config.get('PBoptions', 'workingdir')) and os.path.isdir(config.get('PBoptions', 'workingdir'))):
    os.mkdir(config.get('PBoptions', 'workingdir'))
if not os.path.exists(DEFAULT_BIBLIOGRAPHY):
    a = open(DEFAULT_BIBLIOGRAPHY, 'w')
    a.write(' ')
    a.close()
exclude_by_default = string.split(config.get('PBoptions', 'exclude_by_default'), ',')
PY2EXE = False
if PY2EXE:
    IMGDIR = os.getcwd()
else:
    IMGDIR = GetAppdir()

try:
    from pieberry_pybtex import *
    PYBTEX = True
    print 'Pybtex in use'
except:
    PYBTEX = False
    
try:
    import pynotify
    if pynotify.init(APPNAME):
        PYNOTIFY = True
    else:
        PYNOTIFY = False
except:
    PYNOTIFY = False

ENTRY_TYPE_CHOICES = {0: '@Misc with url field', 
                      1: '@Misc with \url{} in howpublished',
                      2: '@Misc with \href{} in howpublished', 
                      3: '@Online (biblatex only)'
                      }
ENTRY_TYPE_KEYS = {0: 'Misc',
                   1: 'Misc',
                   2: 'Misc',
                   3: 'Online',
                   }

    





### Validators for text inputs
import wx, re, string

class pieUrlValidator(wx.PyValidator):
     """ This validator is used to ensure that the user has entered something
         into the text object editor dialog's text field.
     """
     def __init__(self):
         """ Standard constructor.
         """
         wx.PyValidator.__init__(self)
         # self.regexp = re.compile(r'^http://', re.IGNORECASE)
         self.regexp_long = re.compile("([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}|(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+)(:[0-9]*)?/*[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\),\\\"]")
         self.regexp_short = re.compile("([0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}|(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+)(:[0-9]*)?")

     def Clone(self):
         """ Standard cloner.
             Note that every validator must implement the Clone() method.
         """
         return pieUrlValidator()

     def Validate(self, win=None):
         """ Validate the contents of the given text control.
         """
         textCtrl = self.GetWindow()
         text = textCtrl.GetValue()
         if len(text) == 0:
             # print 'Url required'
             # wx.MessageBox("This must contain a URL", "Error")
             textCtrl.SetBackgroundColour("pink")
             textCtrl.SetFocus()
             textCtrl.Refresh()
             return False
         elif self.regexp_long.match(text) is None:
         # elif (self.regexp_short.match(text) is None) and (self.regexp_long.match(text) is None):
             # print 'Url is invalid'
             # wx.MessageBox("This must contain a URL", "Error")
             textCtrl.SetBackgroundColour("pink")
             textCtrl.SetFocus()
             textCtrl.Refresh()
             return False
         else:
             textCtrl.SetBackgroundColour(
                 wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
             textCtrl.Refresh()
             return True

     def TransferToWindow(self):
         """ Transfer data from validator to window.

             The default implementation returns False, indicating that an error
             occurred.  We simply return True, as we don't do any data transfer.
         """
         return True # Prevent wxDialog from complaining.

     def TransferFromWindow(self):
         """ Transfer data from window to validator.

             The default implementation returns False, indicating that an error
             occurred.  We simply return True, as we don't do any data transfer.
         """
         return True # Prevent wxDialog from complaining.

class piePlainTextValidator(wx.PyValidator):
     def __init__(self):
         wx.PyValidator.__init__(self)
         self.invalidchars = (':', ';', '/', '\\', '>', '<', '|', '@', '!', '*', '^', '`')

     def Clone(self):
         return piePlainTextValidator()

     def Validate(self, win=None):
         """ Validate the contents of the given text control.
         """
         textCtrl = self.GetWindow()
         text = textCtrl.GetValue()
         for ch in self.invalidchars:
             if ch in text: 
                 textCtrl.SetBackgroundColour("pink")
                 textCtrl.SetFocus()
                 textCtrl.Refresh()
                 return False
         textCtrl.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
         textCtrl.Refresh()
         return True

     def TransferToWindow(self):
         return True # Prevent wxDialog from complaining.

     def TransferFromWindow(self):
         return True # Prevent wxDialog from complaining.


class pieBibtexValidator(wx.PyValidator):
     def __init__(self):
         wx.PyValidator.__init__(self)
         self.invalidchars = (r'#', r'$', r'%', r'&') # , r'_') #, '{', '}'
         self.valid_escapes = (r'\#', r'\$', r'\%', r'\&') # , r'\_')
         # self.re_invalidchars = re.compile()
     def Clone(self):
         return pieBibtexValidator()

     def Validate(self, win=None):
         """ Validate the contents of the given text control.
         """
         textCtrl = self.GetWindow()
         text = textCtrl.GetValue()
         fbrack = text.count('{')
         rbrack = text.count('}')
         print fbrack, rbrack
         if fbrack != rbrack:
              textCtrl.SetBackgroundColour("lightblue")
              return False
         # unescaped chars are no longer detected ... this is done automatically
         # at the time of saving. 
         # still look for unbalanced brackets though
         # for ch in self.valid_escapes:
         #      text = string.replace(text, ch, '')
         # for ch in self.invalidchars:
         #     if ch in text: 
         #         textCtrl.SetBackgroundColour("lightblue")
         #         # textCtrl.SetFocus()
         #         # textCtrl.Refresh()
         #         return False
         textCtrl.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
         textCtrl.Refresh()
         return True

     def TransferToWindow(self):
         return True # Prevent wxDialog from complaining.

     def TransferFromWindow(self):
         return True # Prevent wxDialog from complaining.




#!python
##This will work in development on a relative folder basis
##It will then work when installed in site-packages on a target system
##where the runner script is in /usr/bin (or wherever)
##
##So, you don't need anything special - no fancy path tricks.

import pieberry.pieberry
import sys

pieberry.pieberry.main(sys.argv)


# coding=utf-8
# output parsers for pieberry
from pieberry_config import *
import string, re, os, sys
import traceback

class pieAllWriter:
    '''Meta-class that directs different types of output'''
    def __init__(self, entries=[]):
        '''entries is a list of bibdata dicts'''
        self._entries = entries
        self._location = config.get('PBoptions', 'default_bibliography')
        if PYBTEX:
            self._bibtex_writer = piePybtexWriter()
        else:
            self._bibtex_writer = pieBibtexWriter()
        self._tsv_writer = pieTSVWriter()

    def addEntry(self, bibdata):
        '''Add an entry to the output file'''
        self._bibtex_writer.addEntry(bibdata)
        if config.getboolean('PBoptions', 'export_tsv'):
            self._tsv_writer.addEntry(bibdata)

    def setLocation(self, location):
        '''Set the location of the file'''
        self._bibtex_writer.setLocation(location)
        if config.getboolean('PBoptions', 'export_tsv'):
            self._tsv_writer.setLocation(location)

    def write(self):
        '''Write to file. 'types' is a tuple of "contexts" to exclude
        from the write-out process, such as submissions.'''
        self._bibtex_writer.write()
        if config.getboolean('PBoptions', 'export_tsv'):
            self._tsv_writer.write()

class piePybtexWriter:
    '''Class to write out bibliographies as BibTex'''
    def __init__(self, entries=None):
        '''entries is a list of bibdata dicts'''
        self._entries = []
        self._location = config.get('PBoptions', 'default_bibliography')

    def addEntry(self, bibdata):
        '''Add an entry to the output file'''
        self._entries.append(bibdata)

    def setLocation(self, location):
        '''Set the location of the file'''
        self._location = location

    def write(self, backup=True):
        '''Write to file. 'types' is a tuple of "contexts" to exclude
        from the write-out process, such as submissions.'''
        if os.path.exists(self._location):
            try:
                # backup 
                f = open(self._location, 'r')
                if backup:
                    b = open(os.path.join(os.path.split(self._location)[0], 'backup.bib'), 'w')
                    b.write(f.read())
                    b.close()
                f.close()
            except:
                raise 'Could not open file'
        else:
            f = open(self._location, 'w')
            f.write('')
            f.close()
        try:
            from pybtex.database.input import bibtex
            from pybtex.database.output.bibtex import Writer
            from pybtex.database import BibliographyDataError
            from pybtex.core import Entry, Person
            from pybtex.bibtex.utils import split_name_list
            from pieberry_pybtex import pieberry_to_pybtex
        except:
            traceback.print_exc()
            raise 'Could not import pybtex'

        parser = bibtex.Parser()
        bib_object = parser.parse_file(self._location)
        for ent in self._entries:
            if ent['exclude'] == True:
                continue
            new_pybtex_entry = pieberry_to_pybtex(ent, texify=True)
            bib_key = autogen_bibtex_key(ent)
            invalid_key = True
            while invalid_key:
                invalid_key = False
                try:
                    bib_object.add_entry(bib_key, new_pybtex_entry)
                except BibliographyDataError:
                    invalid_key = True
                    bib_key = increment_bibtex_key(bib_key)
        # import latex_codec
        # latex_codec.register()
        writer = Writer(encoding='utf-8')
        writer.write_file(bib_object, self._location)
                    
class pieBibtexWriter:
    '''Class to write out bibliographies as BibTex'''
    def __init__(self, entries=None):
        '''entries is a list of bibdata dicts'''
        self._entries = []
        self._location = config.get('PBoptions', 'default_bibliography')

    def addEntry(self, bibdata):
        '''Add an entry to the output file'''
        self._entries.append(bibdata)

    def setLocation(self, location):
        '''Set the location of the file'''
        self._location = location

    def write(self):
        '''Write to file. 'types' is a tuple of "contexts" to exclude
        from the write-out process, such as submissions.'''
        savedata = ''
        for entry in self._entries:
            if entry['exclude'] == False:
                try:
                    savedata = savedata + format_bibtex_entry(entry)
                except UnicodeDecodeError:
                    print 'Bad unicode - omitting'
                except: 
                    traceback.print_exc()
                    print 'Unknown formatting error'
        if os.path.exists(self._location):
            outfile = open(self._location, 'r')
            current_data = outfile.read()
            outfile.close()
            savedata = current_data + savedata
        outfile = open(self._location, 'w')
        outfile.write(savedata)
        outfile.close()

class pieTSVWriter:
    '''Class to write out bibliographies in TSV with potentially
    arbitrary columns'''
    def __init__(self, entries=None):
        self._entries = []
        self._location = os.path.splitext(config.get('PBoptions', 'default_bibliography'))[0] + '.tsv'

    def addEntry(self, bibdata):
        '''Add an entry to the output file'''
        self._entries.append(bibdata)

    def setLocation(self, location):
        '''Set the location of the file'''
        self._location = os.path.splitext(location)[0] + '.tsv'

    def write(self):
        '''Write to file. 'types' is a tuple of "contexts" to exclude
        from the write-out process, such as submissions.'''
        from pprint import pprint
        lines = []
        header = []
        if os.path.exists(self._location):
            f = open(self._location, 'r')
            lines = [line.rstrip().split('\t') for line in f.readlines()]
            f.close()
        if lines:
            header = lines[0]
        else:
            header = ('bibtex_key', 'author', 'title', 'year', 'month', 'howpublished', 'url', 'final_fn')
            lines.append(header)
        pprint(header)
        for ent in self._entries:
            if ent['exclude'] == True:
                continue
            row = []
            for col in header:
                if ent.has_key(col):
                    row.append(ent[col])
                else:
                    row.append('')
            lines.append(row)
        pprint(lines)
        f = open(self._location, 'w')
        f.writelines(['%s\n' % string.join(line, '\t') for line in lines])
        f.close()

def escape_bad_latex_chars(texstring):
    '''ensure that LaTeX-unsafe characters are escaped'''
    print 'eblc'
    invalidchars = (r'#', r'$', r'%', r'&') # , r'_') #, '{', '}'
    valid_escapes = (r'\#', r'\$', r'\%', r'\&') # , r'\_')
    replace_chars = ('LATEXHASH', 'LATEXDOLLAR', 'LATEXPERCENT', 'LATEXAMPERSAND')
    for k in range(len(invalidchars)):
        ch = valid_escapes[k]
        texstring = string.replace(texstring, ch, replace_chars[k])
    for k in range(len(invalidchars)):
        ch = invalidchars[k]
        texstring = string.replace(texstring, ch, valid_escapes[k])
    for k in range(len(invalidchars)):
        texstring = string.replace(texstring, replace_chars[k], valid_escapes[k])
    return texstring

def unescape_bad_latex_chars(texstring):
    '''get rid of LaTeX-friendly escaping'''
    invalidchars = (r'#', r'$', r'%', r'&') # , r'_') #, '{', '}'
    valid_escapes = (r'\#', r'\$', r'\%', r'\&') # , r'\_')
    for k in range(len(invalidchars)):
        texstring = string.replace(texstring, valid_escapes[k], invalidchars[k])
    return texstring

def untexify(texstring):
    '''remove curly braces and other tex-related stuff'''
    # TODO this is unlikely to work if there's more than one latex macro
    re_tex = re.compile(r'\\.+?\{(.+)\}')
    tex_match = re_tex.search(texstring)
    if tex_match:
        texstring = re_tex.sub(tex_match.group(1), texstring)
    texstring = unescape_bad_latex_chars(texstring)
    transtbl = {ord('{'): None, ord('}'): None}
    texstring = texstring.translate(transtbl)
    return texstring

def format_bibtex_entry(bibdict):
    '''format a single bibtex entry'''
    if bibdict.has_key('bibtex_key'):
        key = bibdict['bibtex_key']
    else:
        key = autogen_bibtex_key(bibdict)
    author = bibdict['author']
    if bibdict.has_key('corporate_author'):
        if bibdict['corporate_author']:
            author = "{%s}" % bibdict['author']
    ent = """
@%s{%s,
  author = {%s},
  title = {%s},
  howpublished = {\href{%s}{On the web}},
  month = {%s},
  year = %s,
  annote = {%s}}

""" % (
        bibdict['bibtex_type'].encode('utf-8'),
        key,
        escape_bad_latex_chars(author).encode('utf-8'),
        escape_bad_latex_chars(bibdict['title']).encode('utf-8'),
        escape_bad_latex_chars(bibdict['howpublished']).encode('utf-8'),
        bibdict['month'].encode('utf-8'),
        bibdict['year'].encode('utf-8'),
        escape_bad_latex_chars(bibdict['annote']).encode('utf-8')
        )
    return ent

def autogen_bibtex_key(bibdict):
    keytitlecompact = string.join([i[:3] for i in string.split(bibdict['title'].encode("utf-8").translate(string.maketrans("",""), string.punctuation)) if len(i) > 3], '') #ouch ... this compacts the first three letters of each word in the title together, to make a dependably unique key
    keyauthorcompact = string.join([i[:1] for i in string.split(bibdict['author'].encode("utf-8").translate(string.maketrans("",""), string.punctuation)) if len(i) > 3], '') #same for authors, but just initials
    key = "%s%s_%s" % (
        keyauthorcompact,
        bibdict['year'].encode('utf-8'), 
        keytitlecompact
        )
    return key

def increment_bibtex_key(keytext):
     '''add digits to end of key to try to avoid identical keys'''
     try:
         last_digit = int(keytext[-1])
         if last_digit == 9:
             key_base = keytext
             last_digit = -1
         else:
             key_base = keytext[:-1]
     except:
         last_digit = -1
         key_base = keytext
     last_digit += 1
     print 'generated key %s%d' % (key_base, last_digit)
     return '%s%d' % (key_base, last_digit)

if __name__ == '__main__':
    import time
    spoofdata = {
        'title': 'Spoof title',
        'author': '{Goldman Saks}',
        'year': '1833',
        'month': 'January',
        'howpublished': 'Mungdinügn',
        'url': 'http://www.themong.spo.nn/a%20b.html',
        'annote': 'notey',
        'ancillary_downloadtime': time.localtime(),
        'ancillary_creationtime': time.localtime(),
        'ancillary_outfilename': 'C:noodles',
        'ancillary_locofdoc': 'http://www.hell.no',
        'exclude': True,
        'internal_author': 'Jon Hoo',
        'internal_title': 'Noh Wai'
        }
    

    w = piePybtexWriter()
    w.addEntry(spoofdata)
    w.write()



#!python

import distutils
import sys, os

std = get_special_folder_path('CSIDL_DESKTOPDIRECTORY')

if argv[1] == '-install':
    os.mkdir(os.path.join(std, 'Pieberry'))
    create_shortcut('pieberrydl', 'Pieberry', os.path.join(std, 'Pieberry', 'pbrun'))

            



#!/bin/python

# for python 2.6
# dependencies - wxpython, beautiful soup, pypdf

import sys, os, os.path, getopt, re, time
import urllib
import urllib2
import string
import traceback
from htmlentitydefs import name2codepoint as n2cp
from BeautifulSoup import *
import urlparse
import wx
import re
from pyPdf import PdfFileWriter, PdfFileReader
from pieberry_config import *
from pieberry_events import *

user_agent = 'Mozilla/Linux'
headers = { 'User-Agent' : user_agent }
re_submission = re.compile(r'^(.+?)(\'s)?( -)? [sS]ubmission')

def wxdate2pydate(date): 
     import datetime 
     assert isinstance(date, wx.DateTime) 
     if date.IsValid(): 
         ymd = map(int, date.FormatISODate().split('-')) 
         return datetime.date(*ymd) 
     else: 
         return None 

def substitute_entity(match):
    ent = match.group(2)
    if match.group(1) == "#":
        return unichr(int(ent))
    else:
        cp = n2cp.get(ent)

        if cp:
            return unichr(cp)
        else:
            return match.group()

def decode_htmlentities(string):
    entity_re = re.compile("&(#?)(\d{1,5}|\w{1,8});")
    return entity_re.subn(substitute_entity, string)[0]

def translate_non_alphanumerics(to_translate, translate_to=u'_'):
    not_letters_or_digits = u'!"#%\'()*+,-./:;<=>?@[\]^_`{|}~'
    if isinstance(to_translate, unicode):
        translate_table = dict((ord(char), unicode(translate_to))
                               for char in not_letters_or_digits)
    else:
        assert isinstance(to_translate, str)
        translate_table = string.maketrans(not_letters_or_digits,
                                           translate_to
                                              *len(not_letters_or_digits))
    return to_translate.translate(translate_table)

def fetch_page(notify_winddow, url):
    '''fetch a page. that is all'''
    try:    
        thepage = urllib2.urlopen(url)
    except ValueError:
        traceback.print_exc()
        wx.PostEvent(notify_window, pieUpdateEvent('warn', "Not a valid or recognised url"))
        raise ValueError
    except urllib2.HTTPError: 
        traceback.print_exc()
        wx.PostEvent(notify_window, pieUpdateEvent('warn', "Could not access website"))
        raise Exception
    thehtml = thepage.read()
    return thehtml

def snarf_pdfs(notify_window, html, url):
    '''get all pdfs on a given page. return list of dicts:
    { pdflink, pdflink_text, pdflink_context } 
    pdflink_context can be one of "submissions" ... '''
    try:
        readup = BeautifulSoup(html)
    except Exception, err:
        traceback.print_exc()
        wx.PostEvent(notify_window, pieUpdateEvent('warn', "Invalid or unparseable HTML"))
        raise Exception
    pdflinks = readup.findAll('a', href=re.compile("^.*\.(PDF|pdf)"))
    if not pdflinks:
        wx.PostEvent(notify_window, pieUpdateEvent('warn', 'No pdf links found'))
        raise Exception
    wx.PostEvent(notify_window, pieUpdateEvent('blank', 'Starting (Found %d files)' % len(pdflinks)))
    head_re = re.compile('[hH][1-7]')
    sub_re = re.compile('[Ss]ubmission')
    retlist = []
    all_are_submissions = ('ubmissions' in readup.title.string)
    for link in pdflinks:
        ret = {
            'pdflink': '',
            'pdflink_text': '',
            'pdflink_context': ''}
        doctext = link.findChildren(text=True)
        nameofdoc = string.join(doctext, " ").decode('utf-8')
        nameofdoc = decode_htmlentities(nameofdoc)
        # Get rid of filename extension
        if nameofdoc[-4:] in ('.pdf', '.PDF', '.Pdf'):
            nameofdoc = nameofdoc[:-4]
        ret['pdflink_text'] = nameofdoc
        # Determine whether this is a submission and flag accordingly
        if sub_re.search(nameofdoc):
            ret['pdflink_context'] = 'submissions'
        # If 'submission' in the last headings do same
        head_h = link.findPrevious(head_re)
        if head_h:
            if sub_re.search(head_h.text):
                ret['pdflink_context'] = 'submissions'
        if all_are_submissions:
             ret['pdflink_context'] = 'submissions'
        # Finally assign relevant link
        # try to work out if the link is already quoted text or not
        if "%" in  link['href'].encode('utf-8'):
            pathstub = link['href'].encode('utf-8')
        else:
            pathstub = urllib.quote(link['href'].encode('utf-8'))
        ret['pdflink'] = urlparse.urljoin(url, pathstub)
        retlist.append(ret)
    return retlist

def download_pdf(notify_window, pdfinfo, author, tag):
    '''download a pdf and return its location.'''
    try:
        tr_fn = translate_non_alphanumerics(pdfinfo['pdflink_text'])[:100] + ".pdf"
        output_directory = os.path.join(config.get('PBoptions', 'workingdir'), author, tag, pdfinfo['pdflink_context'])
        if not os.path.isdir(output_directory): 
            os.makedirs(output_directory)
        existfiles = [fl[9:] for fl in os.listdir(output_directory)] #files with their date prefixes stripped out
        if tr_fn in existfiles: 
            wx.PostEvent(notify_window, pieUpdateEvent('pass', 'Already downloaded', cite=pdfinfo['pdflink_text'], href=pdfinfo['pdflink'], updatelast=True))
            return
        outfilename = os.path.join(output_directory, tr_fn)            
        doc = urllib2.urlopen(pdfinfo['pdflink'])
        docdata = doc.read()
        outfile = open(outfilename, 'wb')
        outfile.write(docdata)
        outfile.close()
        return outfilename
    except Exception, err:
        traceback.print_exc()
        wx.PostEvent(notify_window, pieUpdateEvent('fail', "FAILED to download", cite=pdfinfo['pdflink_text'], href=pdfinfo['pdflink'], updatelast=True))
        raise Exception

def get_internal_pdf_metadata(notify_window, pdfinfo):
    retinfo = {}
    retinfo['creation_date'] = time.localtime()
    retinfo['internal_author'] = ''
    retinfo['internal_title'] = ''
    try:
        pdf_file = file(pdfinfo['outfilename'], 'rb')
        pdfread = PdfFileReader(pdf_file)
        docmetadata = pdfread.getDocumentInfo()
    except:
        traceback.print_exc()
        wx.PostEvent(notify_window, pieUpdateEvent('fail', "Unreadable", cite=pdfinfo['pdflink_text'], href=pdfinfo['outfilename'], updatelast=True))
        pdf_file.close()
        return retinfo
    if docmetadata == None:
        docmetadata = {} #pyPdf appears to send Nonetypes instead of
                         #empty dicts, annoyingly
    # creation date is found here. if not present, fall back to current date
    try:
        if docmetadata.has_key('/CreationDate'):
            rd = docmetadata['/CreationDate'][2:]
            retinfo['creation_date'] = time.strptime("%s %s %s %s %s" % (rd[0:4], rd[4:6], rd[6:8], rd[8:10], rd[10:12]), "%Y %m %d %H %M")
            retinfo['creation_date_guessed'] = False
        else:
            retinfo['creation_date_guessed'] = True
    except: #hack ... but sometimes /creationdate is bunged
        traceback.print_exc()
        retinfo['creation_date_guessed'] = True

    retinfo['download_time'] = time.localtime()
    # some reformatting necessary when author names are computer-inserted
    splre = re.compile("[./_ ]")
    authst = ''
    if not pdfread.documentInfo:
        return retinfo
    if pdfread.documentInfo.author:
        authst = string.join(splre.split(pdfread.documentInfo.author))
        authst = string.capwords(authst)
    retinfo['internal_author'] = authst
    retinfo['internal_title'] = unicode(pdfread.documentInfo.title)
    pdf_file.close()
    return retinfo
        
def gen_title_and_docname(pdfinfo, behaviour):
    '''format the appropriate title and document name'''
    retdata = {}
    if behaviour == 'prepend':
        retdata['title'] = "%s: %s" % (pdfinfo['tag'], pdfinfo['pdflink_text'])
    elif behaviour == 'append_hyphen':
        retdata['title'] = "%s - %s" % (pdfinfo['pdflink_text'], pdfinfo['tag'])
    elif behaviour == 'append_brackets':
        retdata['title'] = "%s (%s)" % (pdfinfo['pdflink_text'], pdfinfo['tag'])
    else:
        retdata['title'] = pdfinfo['pdflink_text']
    final_fn = os.path.join(config.get('PBoptions', 'workingdir'), pdfinfo['author'], pdfinfo['tag'], pdfinfo['pdflink_context'], "%s %s" % (time.strftime("%Y%m%d", pdfinfo['creation_date']), os.path.basename(pdfinfo['outfilename'])))
    retdata['final_fn'] = final_fn
    return retdata

# def scrapeme(url, config, tag='', author='', prefix='', postfix=None, usepostfix=True, notify_window=None):
#     """Read the page and find all pdf links contained within"""
#     try:    
#         thepage = urllib2.urlopen(url)
#     except ValueError:
#         traceback.print_exc()
#         wx.PostEvent(notify_window, pieUpdateEvent('warn', "Not a valid or recognised url"))
#         return
#     except urllib2.HTTPError: 
#         traceback.print_exc()
#         return 

#     thehtml = thepage.read()
#     try:
#         readup = BeautifulSoup(thehtml)
#     except Exception, err:
#         traceback.print_exc()
#         wx.PostEvent(notify_window, pieUpdateEvent('warn', "Invalid or unparseable HTML"))
#         return
#     #TODO use urlparse.urljoin() 

#     thebib = {}
#     pdflinks = readup.findAll('a', href=re.compile("^.*\.(PDF|pdf)"))
#     wx.PostEvent(notify_window, pieUpdateEvent('blank', 'Starting (Found %d files)' % len(pdflinks)))
#     num_pdf_links = len(pdflinks)
#     curr_pdf_link = 0

#     for pdflink in pdflinks:
#         curr_pdf_link += 1
#         bibdata = {}
#         doctext = pdflink.findChildren(text=True)
#         nameofdoc = string.join(doctext, " ").decode('utf-8')
#         nameofdoc = decode_htmlentities(nameofdoc)
#         if '.pdf' in nameofdoc: #in case the web site is quoting full file name
#             nameofdoc = string.split(nameofdoc, '.pdf')[0]
#         if '.PDF' in nameofdoc: #in case the web site is quoting full file name
#             nameofdoc = string.split(nameofdoc, '.PDF')[0]
#         #use page's title bar as postfix if no explicit postfix declared
#         if usepostfix and not postfix: 
#             postfix = readup.title.string.split(' - ')[0].strip()
#             nameofdoc = nameofdoc + ' - ' + postfix
#         elif usepostfix and postfix:
#             nameofdoc = nameofdoc + ' - ' + postfix

#         # try to work out if the link is already quoted text or not
#         if "%" in  pdflink['href'].encode('utf-8'):
#             pathstub = pdflink['href'].encode('utf-8')
#         else:
#             pathstub = urllib.quote(pdflink['href'].encode('utf-8'))

#         locofdoc = urlparse.urljoin(url, pathstub)
#         bibdata['ancillary_locofdoc'] = locofdoc
#         bibdata['title'] = nameofdoc
#         #we want to be able to navigate back to the document in LaTeX
#         #with the aid of hyperref
#         bibdata['howpublished'] = "\href{%s}{Web site}" % string.replace(locofdoc, '%', r'\%')

#         try:
#             wx.PostEvent(notify_window, pieUpdateEvent('warn', '[%d/%d] Downloading' % (curr_pdf_link, num_pdf_links), cite=bibdata['title'], href=locofdoc))
#             tr_fn = translate_non_alphanumerics(nameofdoc)[:100] + ".pdf"
#             # if os.path.exists(outfilename): continue # don't download twice
#             existfiles = [fl[9:] for fl in os.listdir(os.path.join(config.get('PBoptions', 'workingdir'), author, tag))] #files with their date prefixes stripped out
#             if tr_fn in existfiles: 
#                 wx.PostEvent(notify_window, pieUpdateEvent('pass', 'Already downloaded', cite=bibdata['title'], href=locofdoc, updatelast=True))
#                 continue #don't download twice

#             outfilename = os.path.join(config.get('PBoptions', 'workingdir'), author, tag, tr_fn)            
#             doc = urllib2.urlopen(locofdoc)
#             docdata = doc.read()
#             outfile = open(outfilename, 'wb')
#             outfile.write(docdata)
#             outfile.close()
#         except Exception, err:
#             traceback.print_exc()
#             wx.PostEvent(notify_window, pieUpdateEvent('fail', "FAILED to download", cite=bibdata['title'], href=locofdoc, updatelast=True))
#             continue

#         wx.Yield()

#         # Move to reading the pdf's data and get appropriate metadata
#         try:
#             pdf_file = file(outfilename, 'rb')
#             pdfread = PdfFileReader(pdf_file)
#             docmetadata = pdfread.getDocumentInfo()
#         except:
#             wx.PostEvent(notify_window, pieUpdateEvent('fail', "FAILED unreadable", cite=bibdata['title'], href=outfilename, updatelast=True))
#             continue

#         if docmetadata == None:
#             docmetadata = {} #pyPdf appears to send Nonetypes instead
#                              #of empty dicts, annoyingly

#         bibdata['annote'] = 'Downloaded on: %s\n' % time.ctime()
#         # If there's internal data for creation date, use that for the
#         # bibliography. Otherwise, use the current date.
#         if docmetadata.has_key('/CreationDate'):
#             rd = docmetadata['/CreationDate'][2:]
#             encdate = time.strptime("%s %s %s %s %s" % (rd[0:4], rd[4:6], rd[6:8], rd[8:10], rd[10:12]), "%Y %m %d %H %M")
#             bibdata['year'] = unicode(time.strftime("%Y", encdate))
#             bibdata['month'] = unicode(time.strftime("%B", encdate))
#         else:
#             bibdata['year'] = unicode(time.strftime("%Y"))
#             bibdata['month'] = unicode(time.strftime("%B"))
#             bibdata['annote'] = bibdata['annote'] + 'Creation date was unavailable - used date of download\n'
#             encdate = time.localtime()
#         bibdata['ancillary_downloadtime'] = time.localtime()
#         bibdata['ancillary_creationtime'] = encdate

#         # If we've not set a global author for these docs, try to
#         # figure it out from the metadata
#         bibdata['author'] = "{" + author + "}"
#         if docmetadata.has_key('/Author'):
#             splre = re.compile("[./ ]")
#             authst = string.join(splre.split(docmetadata['/Author']))
#             authst = string.capwords(authst)
#             if author == "Unknown":
#                 bibdata['author'] = authst
#             else:
#                 bibdata['annote'] = bibdata['annote'] + 'Auth_from_metadata: ' + authst + '\n'

#         #rather dodgy assumption that submissions are always titled 'person's submission ... '
#         submatch = re_submission.match(nameofdoc)
#         if submatch:
#             bibdata['author'] = "{%s}" % submatch.group(1)

#         if docmetadata.has_key('/Title'):
#             bibdata['annote'] = bibdata['annote'] + 'Title_from_metadata: %s\n' % docmetadata['/Title']
        
#         #append file creation date digits to filename to aid archival
#         pdf_file.close()
#         final_fn = os.path.join(config.get('PBoptions', 'workingdir'), author, tag, "%s %s" % (time.strftime("%Y%m%d", encdate), tr_fn))
#         try:
#             os.rename(outfilename, final_fn)
#             bibdata['annote'] = bibdata['annote'] + 'Saved to: %s\n' % final_fn
#             bibdata['ancillary_outfilename'] = final_fn
#         except:
#             traceback.print_exc()
#             print 'Could not rename file'
#             bibdata['annote'] = bibdata['annote'] + 'Saved to: %s\n' % outfilename
#             bibdata['ancillary_outfilename'] = outfilename

#         data_id = wx.NewId()
#         thebib[data_id] = bibdata
#         wx.PostEvent(notify_window, pieUpdateEvent('success', 'Success', cite="%s (%s). %s" % (bibdata['author'][1:-1], bibdata['year'], bibdata['title']), href=locofdoc, data_id=data_id, updatelast=True))

#     return thebib

# def autogen_bibtex_key(bibdict):
#     keytitlecompact = string.join([i[:3] for i in string.split(bibdict['title'].encode("utf-8").translate(string.maketrans("",""), string.punctuation)) if len(i) > 3], '') #ouch ... this compacts the first three letters of each word in the title together, to make a dependably unique key
#     keyauthorcompact = string.join([i[:1] for i in string.split(bibdict['author'].encode("utf-8").translate(string.maketrans("",""), string.punctuation)) if len(i) > 3], '') #same for authors, but just initials
#     key = "%s%s_%s" % (
#         keyauthorcompact,
#         bibdict['year'].encode('utf-8'), 
#         keytitlecompact
#         )
#     return key

# def increment_bibtex_key(keytext):
#      '''add digits to end of key to try to avoid identical keys'''
#      try:
#           last_digit = int(keytext[-1])
#           if last_digit == 9:
#                key_base = keytext
#                last_digit = 0
#           else:
#                key_base = keytext[:-1]
#      except:
#           last_digit = 0 
#           key_base = keytext
#           last_digit += 1
#      return '%s%d' % (key_base, last_digit)

# def format_bibtex_entry(bibdict):
#     '''format a single bibtex entry'''
#     if bibdict.has_key('bibtex_key'):
#         key = bibdict['bibtex_key']
#     else:
#         key = autogen_bibtex_key(bibdict)
#     ent = """
# @Misc{%s,
#   author = {%s},
#   title = {%s},
#   howpublished = {%s},
#   month = {%s},
#   year = %s,
#   annote = {%s}}

# """ % (
#         key,
#         bibdict['author'].encode('utf-8'),
#         bibdict['title'].encode('utf-8'),
#         bibdict['howpublished'].encode('utf-8'),
#         bibdict['month'].encode('utf-8'),
#         bibdict['year'].encode('utf-8'),
#         bibdict['annote'].encode('utf-8')
#         )
#     return ent


def prefetch_page_info(notify_window, url):
    '''pre-fetch a page header to assist determing pre/postfix'''
    try:    
        wx.PostEvent(notify_window, pieUpdateEvent('pass', "Reading..."))
        if sys.version_info >= (2,6):
            thepage = urllib2.urlopen(url, data=None, timeout=1) 
        else:
            thepage = urllib2.urlopen(url, data=None) 
        # timeout is 1s, we don't want to hang the UI waiting 
    except ValueError:
        traceback.print_exc()
        wx.PostEvent(notify_window, pieUpdateEvent('warn', "Not a valid or recognised url", updatelast=True))
        return 
    except: 
        wx.PostEvent(notify_window, pieUpdateEvent('warn', "Not a valid or recognised url", updatelast=True))
        traceback.print_exc()
        return 
    thehtml = thepage.read()
    try:
        readup = BeautifulSoup(thehtml)
    except Exception, err:
        traceback.print_exc()
        wx.PostEvent(notify_window, pieUpdateEvent('warn', "Invalid or unparseable HTML", updatelast=True))
        return 
    tag = translate_non_alphanumerics(readup.title.string.split(' - ')[0][:90].strip())
    wx.PostEvent(notify_window, pieUpdateEvent('pass', "Read.", updatelast=True))
    return tag

    




# Copyright (C) 2006, 2007, 2008, 2009  Andrey Golovizin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re

from pybtex.style.formatting import FormatterBase, toplevel
from pybtex.style.template import (
    join, words, field, optional, first_of,
    names, sentence, tag, optional_field
)
from pybtex.richtext import Text, Symbol

def dashify(text):
    dash_re = re.compile(r'-+')
    return Text(Symbol('ndash')).join(dash_re.split(text))

pages = field('pages', apply_func=dashify)

date = words [optional_field('month'), field('year')]

class Formatter(FormatterBase):
    def format_names(self, role):
        return sentence(capfirst=False) [names(role, sep=', ', sep2 = ' and ', last_sep=', and ')]

    def format_article(self, e):
        volume_and_pages = first_of [
            join [field('volume'), optional [':', pages]],
            words ['pages', optional [pages]]
        ]
        template = toplevel [
            self.format_names('author'),
            sentence [field('title')],
            sentence [
                tag('emph') [field('journal')], volume_and_pages, date],
        ]
        return template.format_data(e)
        
    def format_author_or_editor(self, e):
        if e.persons['author']:
            return self.format_names('author')
        else:
            editors = self.format_names('editors')
            if len(e.persons['editors']) > 1:
                word = 'editors'
            else:
                word = 'editor'
            return words [editors, word]
    
    def format_volume_and_series(self, e):
        volume_and_series = optional [
            sentence(capfirst=False, sep=' ') [
                'Volume', field('volume'), optional [
                    words ['of', field('series')]
                ]
            ]
        ]
        number_and_series = optional [
            sentence(capfirst=False, sep=' ') [
                join(sep=Symbol('nbsp')) ['Number', field('number')],
                optional [
                    words ['in', field('series')]
                ]
            ]
        ]
        series = optional [ sentence(capfirst=False) [field('series')] ]
        return first_of [
                volume_and_series,
                number_and_series,
                series,
            ]
    
    def format_chapter_and_pages(self, e):
        return join(sep=', ') [
            optional [words ['chapter', field('chapter')]],
            optional [words ['pages', pages]],
        ]

    def format_people(self, e):
        template = toplevel [ 
            self.format_author_or_editor(e)
            ]
        return template.format_data(e)

    def format_book(self, e):
        template = toplevel [
            self.format_author_or_editor(e),
            tag('emph') [sentence [field('title')]],
            self.format_volume_and_series(e),
            sentence [field('publisher'), date],
        ]
        return template.format_data(e)

    def format_booklet(self, e):
        template = toplevel [
            sentence [self.format_names('author')],
            sentence [tag('emph') [field('title')]],
            sentence [
                optional_field('howpublished'),
                optional_field('address'),
                date,
                optional_field('note'),
            ]
        ]
        return template.format_data(e)

    def format_misc(self, e):
        return self.format_booklet(e)

    def format_Misc(self, e):
        return self.format_booklet(e)

    def format_other(self, e):
        return self.format_booklet(e)

    def format_booklet_with_url(self, e):
        template = toplevel [
            sentence [self.format_names('author')],
            sentence [tag('emph') [field('title')]],
            sentence [
                optional_field('howpublished'),
                optional_field('address'),
                date,
                optional_field('note'),
            ],
            optional_field('url')
        ]
        return template.format_data(e)

    def unmung_hrefs(self, field):
        import re
        print type(field)
        re_href = re.compile(r'\href\{(.*)\}\{Web [sS]ite\}')
        m = re_href.match(field)
        if m:
            return m.group(1)
        return field

    def format_inbook(self, e):
        template = toplevel [
            sentence [self.format_names('author')],
            sentence [
                tag('emph') [field('title')],
                self.format_chapter_and_pages(e),
            ],
            self.format_volume_and_series(e),
            sentence [
                field('publisher'),
                optional_field('address'),
                optional [
                    words [field('edition'), 'edition']
                ],
                date,
                optional_field('note'),
            ]
        ]
        return template.format_data(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# generated by wxGlade 0.6.3 on Wed Jun 30 23:06:46 2010

import wx
from pieberry_validators import *
from pieberry_config import *
# begin wxGlade: extracode
# end wxGlade



class pieBibEditDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        # begin wxGlade: pieBibEditDialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)
        self.panel_4 = wx.Panel(self, -1)
        self.panelx = wx.Panel(self, -1, style=wx.NO_BORDER|wx.TAB_TRAVERSAL)
        self.label_a = wx.StaticText(self.panelx, -1, "BibTeX key")
        self.keyCtrl = wx.TextCtrl(self.panelx, -1, "", validator=pieBibtexValidator())
        self.keyAutoCb = wx.CheckBox(self.panelx, -1, label="Auto-generate")
        self.label_3 = wx.StaticText(self.panelx, -1, "Author:", style=wx.EXPAND)
        self.authorIsCorporateCb = wx.CheckBox(self.panelx, -1, label="Author is corporate entity", style=wx.CHK_3STATE|wx.CHK_ALLOW_3RD_STATE_FOR_USER)
        self.authorCtrl = wx.TextCtrl(self.panelx, -1, "", style=wx.TE_PROCESS_ENTER, validator=pieBibtexValidator())
        self.authorSwapButton = wx.BitmapButton(self.panelx, -1, wx.ArtProvider.GetBitmap(wx.ART_GO_BACK, wx.ART_TOOLBAR, (16, 16)))
        self.authorAltCtrl = wx.TextCtrl(self.panelx, -1, "", style=wx.TE_READONLY)

        self.label_4 = wx.StaticText(self.panelx, -1, "Title:")
        self.titleCtrl = wx.TextCtrl(self.panelx, -1, "", style=wx.TE_PROCESS_ENTER|wx.EXPAND, validator=pieBibtexValidator())
        self.titleSwapButton = wx.BitmapButton(self.panelx, -1, wx.ArtProvider.GetBitmap(wx.ART_GO_BACK, wx.ART_TOOLBAR, (16, 16)))
        self.titleAltCtrl = wx.TextCtrl(self.panelx, -1, "", style=wx.TE_READONLY)


        self.label_5 = wx.StaticText(self.panelx, -1, "Date of publication:")
        self.datePicker = wx.DatePickerCtrl(self.panelx, -1)
        self.label_6 = wx.StaticText(self.panelx, -1, "How published:")
        self.howpublishedCtrl = wx.TextCtrl(self.panelx, -1, "", style=wx.TE_PROCESS_ENTER, validator=pieBibtexValidator())
        self.label_7 = wx.StaticText(self.panelx, -1, "Note:")
        self.noteCtrl = wx.TextCtrl(self.panelx, -1, "", style=wx.TE_MULTILINE, validator=pieBibtexValidator())
        self.label_8 = wx.StaticText(self.panelx, -1, "Annotation:")
        self.annoteCtrl = wx.TextCtrl(self.panelx, -1, "", style=wx.TE_MULTILINE, validator=pieBibtexValidator())
        self.btCancel = wx.Button(self.panel_4, wx.ID_CANCEL, "Cancel")
        self.btOk = wx.Button(self.panel_4, -1, "Ok")


        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.onCancel, self.btCancel)
        self.Bind(wx.EVT_BUTTON, self.onOk, self.btOk)
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: pieBibEditDialog.__set_properties
        self.SetTitle("Edit Bibliography Entry")
        self.SetSize((547, 558))
        self.panel_4.SetMinSize((55, 35))
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: pieBibEditDialog.__do_layout
        sizer_7 = wx.BoxSizer(wx.VERTICAL)
        sizer_8 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_9 = wx.BoxSizer(wx.VERTICAL)
        sizer_A = wx.BoxSizer(wx.HORIZONTAL)
        sizer_B = wx.BoxSizer(wx.HORIZONTAL)
        sizer_C = wx.BoxSizer(wx.HORIZONTAL)
        sizer_D = wx.BoxSizer(wx.HORIZONTAL)
        sizer_A.Add(self.label_a, 0, wx.ALL, 3)
        sizer_A.Add(self.keyCtrl, 1, wx.ALL|wx.EXPAND, 3)
        sizer_A.Add(self.keyAutoCb, 1, wx.ALL, 3)
        sizer_9.Add(sizer_A, 0, wx.EXPAND)
        sizer_9.Add(wx.StaticLine(self.panelx, style=wx.LI_HORIZONTAL|wx.EXPAND))
        sizer_D.Add(self.label_3, 1, wx.ALL, 3)
        sizer_D.Add(self.authorIsCorporateCb, 0, wx.ALL, 3)
        sizer_9.Add(sizer_D, 0, wx.EXPAND)
        sizer_B.Add(self.authorCtrl, 1, wx.ALL|wx.EXPAND, 3)
        sizer_B.Add(self.authorSwapButton, 0, wx.ALL, 3)
        sizer_B.Add(self.authorAltCtrl, 1, wx.ALL|wx.EXPAND, 3)
        sizer_9.Add(sizer_B, 0, wx.EXPAND)
        sizer_9.Add(self.label_4, 0, wx.ALL, 3)
        sizer_C.Add(self.titleCtrl, 1, wx.ALL|wx.EXPAND, 3)
        sizer_C.Add(self.titleSwapButton, 0, wx.ALL, 3)
        sizer_C.Add(self.titleAltCtrl, 1, wx.ALL|wx.EXPAND, 3)
        sizer_9.Add(sizer_C, 0, wx.EXPAND)
        sizer_9.Add(self.label_5, 0, wx.ALL, 3)
        sizer_9.Add(self.datePicker, 0, wx.ALL, 3)
        sizer_9.Add(self.label_6, 0, wx.ALL, 3)
        sizer_9.Add(self.howpublishedCtrl, 0, wx.ALL|wx.EXPAND, 3)
        sizer_9.Add(self.label_7, 0, 0, 0)
        sizer_9.Add(self.noteCtrl, 1, wx.ALL|wx.EXPAND, 3)
        sizer_9.Add(self.label_8, 0, 0, 0)
        sizer_9.Add(self.annoteCtrl, 1, wx.ALL|wx.EXPAND, 3)
        self.panelx.SetSizer(sizer_9)
        sizer_7.Add(self.panelx, 6, wx.ALL|wx.EXPAND, 3)
        sizer_8.Add((20, 20), 3, wx.EXPAND, 0)
        sizer_8.Add(self.btCancel, 0, wx.ALL, 3)
        sizer_8.Add(self.btOk, 0, wx.ALL, 3)
        self.panel_4.SetSizer(sizer_8)
        sizer_7.Add(self.panel_4, 0, wx.EXPAND, 0)
        self.SetSizer(sizer_7)
        self.Layout()
        # end wxGlade

    def onCancel(self, event): # wxGlade: pieBibEditDialog.<event_handler>
        print "Event handler `onCancel' not implemented!"
        event.Skip()

    def onOk(self, event): # wxGlade: pieBibEditDialog.<event_handler>
        print "Event handler `onOk' not implemented!"
        event.Skip()

# end of class pieBibEditDialog


class pieConfigDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        # begin wxGlade: pieConfigDialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)
        self.panel_2 = wx.Panel(self, -1)
        self.panel_1 = wx.Panel(self, -1)
        self.label_1 = wx.StaticText(self.panel_1, -1, "Root directory for library:")
        self.workDirCtrl = wx.TextCtrl(self.panel_1, -1, "", style=wx.TE_READONLY)
        self.workDirBt = wx.Button(self.panel_1, -1, "Select")
        self.label_2 = wx.StaticText(self.panel_1, -1, "Principal bibliography file:")
        self.mainBibCtrl = wx.TextCtrl(self.panel_1, -1, "", style=wx.TE_READONLY)
        self.mainBibBt = wx.Button(self.panel_1, -1, "Select")
        self.tsvBibCb = wx.CheckBox(self.panel_1, -1, label='Also export to tab-separated-values spreadsheet')
        self.defaultEntryTypeChoice = wx.Choice(self.panel_1, -1, choices=ENTRY_TYPE_CHOICES.values())
        self.okBt = wx.Button(self.panel_2, -1, "Ok")
        self.cancelBt = wx.Button(self.panel_2, wx.ID_CANCEL, "Cancel")

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.onSelectRoot, self.workDirBt)
        self.Bind(wx.EVT_BUTTON, self.onSelectBib, self.mainBibBt)
        self.Bind(wx.EVT_BUTTON, self.onOk, self.okBt)
        self.Bind(wx.EVT_BUTTON, self.onCancel, self.cancelBt)
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: pieConfigDialog.__set_properties
        self.SetTitle("PieBerry Settings")
        self.panel_2.SetMinSize((532, 34))
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: pieConfigDialog.__do_layout
        sizer_2 = wx.BoxSizer(wx.VERTICAL)
        sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_4 = wx.BoxSizer(wx.VERTICAL)
        sizer_6 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_4.Add(self.label_1, 0, wx.ALL, 5)
        sizer_5.Add(self.workDirCtrl, 2, wx.ALL, 5)
        sizer_5.Add(self.workDirBt, 0, wx.ALL, 5)
        sizer_4.Add(sizer_5, 1, wx.EXPAND, 0)
        sizer_4.Add(self.label_2, 0, wx.ALL, 5)
        sizer_6.Add(self.mainBibCtrl, 2, wx.ALL, 5)
        sizer_6.Add(self.mainBibBt, 0, wx.ALL, 5)
        sizer_4.Add(sizer_6, 1, wx.EXPAND, 0)
        sizer_4.Add(self.tsvBibCb, 0, wx.EXPAND|wx.ALL, 5)
        sizer_a = wx.BoxSizer(wx.HORIZONTAL)
        sizer_a.Add(wx.StaticText(self.panel_1, -1, 'Default BibTex style:'), 1, wx.EXPAND|wx.ALL, 5)
        sizer_a.Add(self.defaultEntryTypeChoice, 0, wx.ALL, 5)
        sizer_4.Add(sizer_a, 1, wx.EXPAND, 0)
        self.panel_1.SetSizer(sizer_4)
        sizer_2.Add(self.panel_1, 5, wx.EXPAND, 0)
        sizer_3.Add((20, 20), 3, 0, 0)
        sizer_3.Add(self.okBt, 0, wx.ALL, 3)
        sizer_3.Add(self.cancelBt, 0, wx.ALL, 3)
        sizer_3.Add((5, 5), 0, 0, 0)
        self.panel_2.SetSizer(sizer_3)
        sizer_2.Add(self.panel_2, 0, wx.EXPAND, 0)
        self.SetSizer(sizer_2)
        sizer_2.Fit(self)
        self.Layout()
        # end wxGlade

    def onSelectRoot(self, event): # wxGlade: pieConfigDialog.<event_handler>
        print "Event handler `onSelectRoot' not implemented"
        event.Skip()

    def onSelectBib(self, event): # wxGlade: pieConfigDialog.<event_handler>
        print "Event handler `onSelectBib' not implemented"
        event.Skip()

    def onOk(self, event): # wxGlade: pieConfigDialog.<event_handler>
        print "Event handler `onOk' not implemented"
        event.Skip()

    def onCancel(self, event): # wxGlade: pieConfigDialog.<event_handler>
        print "Event handler `onCancel' not implemented"
        event.Skip()

# end of class pieConfigDialog


if __name__ == "__main__":
    app = wx.PySimpleApp(0)
    wx.InitAllImageHandlers()
    dialog_1 = pieConfigDialog(None, -1, "")
    app.SetTopWindow(dialog_1)
    dialog_1.Show()
    app.MainLoop()



# Source:
# http://groups.google.com/group/wxPython-users/browse_thread/thread/5c66b0d1b1d438a2
# Robin Dunn 
# Software Craftsman 
# http://wxPython.org 

# [   htmldataobj.py 2K ] 

import wx 

class HTMLDataObject(wx.PyDataObjectSimple): 
    header = """Version:0.9 
StartHTML:%08d 
EndHTML:%08d 
StartFragment:%08d 
EndFragment:%08d 
SourceURL:%s 
""" 
    startFrag = """<html><body> 
<!--StartFragment-->""" 
    endFrag = """<!--EndFragment--> 
</body> 
</html> 
""" 
    def __init__(self): 
        fmt = wx.CustomDataFormat("HTML Format") 
        wx.PyDataObjectSimple.__init__(self, fmt) 
        self.value = None 
        self.fmt = fmt 
    def SetValue(self, html, srcURL=""): 
        if isinstance(html, unicode): 
            fragment = html.encode('utf-8') 
        else: 
            # This assumes that it's already utf-8 or plain ascii and 
            # not some other encoding 
            fragment = html 
        # Assemble the text using the templates above, and the html 
        # fragment given to us.  First calculate the sizes that will 
        # be put in the header. 
        startHtml = len(self.header) + 4*4 + len(srcURL) 
        # 4*4 is the number of extra digits that will be substituted 
        # for all the %08d items in the header 
        startFragment = startHtml + len(self.startFrag) 
        endFragment = startFragment + len(fragment) 
        endHtml = endFragment + len(self.endFrag) 
        value = self.header % (startHtml, endHtml, startFragment, endFragment, srcURL) 
        value += self.startFrag + fragment + self.endFrag 
        self.value = value 
    def GetValue(self): 
        # TODO: This should extract just the fragment from the whole 
        # value and return that, in order to be consistent with how 
        # SetValue works. 
        return self.value 
    # Overridden base class methods that implement the data transfer 
    def GetDataSize(self): 
        print 'GetDataSize' 
        return len(self.value) 
    def GetDataHere(self): 
        print 'GetDataHere' 
        return self.value 
    def SetData(self, buf): 
        print 'SetData' 
        self.value = buf 


if __name__ == '__main__': 
    def copy(evt): 
        do = HTMLDataObject() 
        do.SetValue("<b>hello world</b>") 
        wx.TheClipboard.Open() 
        wx.TheClipboard.SetData(do) 
        wx.TheClipboard.Close() 
    def paste(evt): 
        do = HTMLDataObject() 
        wx.TheClipboard.Open() 
        if not wx.TheClipboard.IsSupported(do.fmt): 
            print "expected format not in the clipboard" 
        else: 
            wx.TheClipboard.GetData(do) 
        wx.TheClipboard.Close() 
        print do.GetValue() 
    app = wx.App(False) 
    frm = wx.Frame(None) 
    cbtn = wx.Button(frm, -1, "copy") 
    cbtn.Bind(wx.EVT_BUTTON, copy) 
    pbtn = wx.Button(frm, -1, "paste") 
    pbtn.Bind(wx.EVT_BUTTON, paste) 
    sizer = wx.BoxSizer() 
    sizer.Add(cbtn, 0, wx.ALL, 5) 
    sizer.Add(pbtn, 0, wx.ALL, 5) 
    frm.SetSizer(sizer) 
    frm.Fit() 
    frm.Show() 
    app.MainLoop() 




from distutils.core import setup
import os
from pprint import pprint

ics = [os.path.join('pieberry', i) for i in os.listdir('pieberry') if os.path.splitext(i)[1] == '.png']

try:
    import py2exe
    imfiles = ('.', ics)
    # origIsSystemDLL = py2exe.build_exe.isSystemDLL
    # def isSystemDLL(pathname):
    #     if os.path.basename(pathname).lower() in ("msvcp71.dll", "dwmapi.dll"):
    #         return 0
    #     return origIsSystemDLL(pathname)
    # py2exe.build_exe.isSystemDLL = isSystemDLL
except:
    imfiles = ('pieberry', ics)

files=[('.', ['pieberry.ico', 'pieberry32.ico']), imfiles]

#manifest for py2exe
manifest = """
 <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <assembly xmlns="urn:schemas-microsoft-com:asm.v1"
 manifestVersion="1.0">
 <assemblyIdentity
     version="0.64.1.0"
     processorArchitecture="x86" 
     name="Controls"
     type="win32"
 />
 <description>Your Application</description>
 <dependency>
     <dependentAssembly>
         <assemblyIdentity
             type="win32"
             name="Microsoft.Windows.Common-Controls"
             version="6.0.0.0"
             processorArchitecture="X86"
             publicKeyToken="6595b64144ccf1df"
             language="*"
         />
     </dependentAssembly>
 </dependency>
 </assembly>
 """

classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Science/Research',
    'Environment :: Win32 (MS Windows)',
    'Environment :: X11 Applications :: GTK',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Natural Language :: English',
    'Programming Language :: Python :: 2.5',
    'Topic :: Text Processing :: Markup :: LaTeX',
    'Operating System :: OS Independent'
    ]


opts = {
    "py2exe": {
        "includes": "BeautifulSoup, pyPdf",
#        "dll_excludes": ["MSVCP90.dll"]
        }
    }

setup(name = "pieberry-library-assistant",
      version = "1.6-1",
      description = "A program to download pdf documents from public websites, and catalogue them in BibTeX format",
      author = "Raif Sarcich",
      author_email = "raifsarcich@gmail.com",
      url = "none",
      download_url = 'http://members.iinet.net.au/~raifsarcich/pieberry/',
      classifiers = classifiers,
      license = 'GNU General public licence',
      #Name the folder where your packages live:
          #(If you have other packages (dirs) or modules (py files) then
      #put them into the package directory - they will be found 
      #recursively.)
      packages = ['pieberry'],
      #'package' package must contain files (see list above)
      #I called the package 'package' thus cleverly confusing the whole issue...
      #This dict maps the package name =to=> directories
      #It says, package *needs* these files.
      package_data = {'pieberry' : ['*.png'] },
      install_requires = ['pyPdf', 'BeautifulSoup', 'pybtex'],
      # data_files = [("pieberry", "pieberry/*.png"), (".", "*.png")],
      #'runner' is in the root.
      scripts = ["pieberrydm", 'pieberrydm.pyw', 'postinstall.py'],
      long_description = """PIEBERRY (IT'S FOR YOUR LIBRARY)

This is a program which I wrote to automate a painful aspect of my
work-life - downloading, storing, cataloguing and referencing
documents from (mainly public sector & government) websites.

These websites publish reams of documents in pdf format, usually in a
random range of cryptic CMS-generated filename schemas, with
incomplete or non-existent file metadata. 

Typically I download these, rename them with an intelligible title, a
six-digit archival date prefix, store then in an appropriate folder,
and enter them into my database of reference materials for use with
LaTeX/BibTex.

Actually, scratch that. What I REALLY do is download them, leave them
on my Desktop folder, look at them once, forget them, fill up my disk
quota, delete them, realise I've lost them and download them all over
again.

Hence, Pieberry, which will do all of the good and none of the bad
described above.

It's mainly for my use, but I hope that someone else will find it
useful. I'm open to requests for features.

It's written in Python, with the PortablePython 2.5.4 distribution
(which contains wxpython) in mind, but also requires Beautiful Soup,
pyPdf and Pybtex, which are all available through easy_install from
setuptools.""",
      windows = [{
            "script": 'pieberrydm.pyw',
            #"console": 'pieberrydm.pyw',
            "icon_resources": [(0, 'pieberry32.ico')],
            "other_resources": [(24, 1, manifest)]
            }],
      data_files=files,#[('.', ['pieberry.ico', 'pieberry32.ico']),('pieberry',ics)],
      options = opts
    #
    #This next part it for the Cheese Shop, look a little down the page.
    #classifiers = []     
) 



# 

from pieberry_config import *
from pybtex.database.input import bibtex
from pieberry_pybtex_style import Formatter
from pybtex.richtext import Text, Tag
from pybtex.backends import latex, html, plaintext
from pybtex.bibtex.utils import split_name_list
from pybtex.core import Entry, Person
from StringIO import StringIO

import re
# import latex_codec

# latex_codec.register()

re_href = re.compile(r'\\href\{(.+?)\}\{(.+?)\}')
re_fileloc = re.compile(r'Saved[_ ]to: (.+?\.(pdf|PDF|Pdf))')
re_corpname = re.compile(r'^\{(.+?)\}(.*)')
re_search = re.compile(r'\s*(title|t|author|a|month|m|year|y|key|k|all)==(.+?)\s*(?:,|$)')
search_terms = ('title', 't', 'author', 'a', 'month', 'm', 'year', 'y', 'key', 'k', 'all')
search_abbrev = {'t': 'title', 'a': 'author', 'y': 'year', 'm': 'month', 'k': 'key', 'all': 'all'}
bibtex_fields = ('title', 'howpublished', 'month', 'year', 'annote', 'note', 'url', 'volume', 'number', 'pages', 'journal', 'edition', 'series')
person_fields = ('author', 'editor')

def pieberry_to_pybtex(ent, texify=False):
    '''convert from pieberry's data dict to a pybtex Entry'''
    from pieberry_output import escape_bad_latex_chars as eblc
    if ent.has_key('bibtex_type'):
        new_pybtex_entry = Entry(ent['bibtex_type'].lower())
    else:
        new_pybtex_entry = Entry(config.get('PBoptions', 'default_bibtex_entry_type'))
    for k, v in ent.items():
        if k in person_fields:
            if ent.has_key('corporate_author') and ent['corporate_author'] == True:
                new_pybtex_entry.add_person(Person('{%s}' % ent[k]), k)
            else:
                for name in split_name_list(ent[k]):
                    new_pybtex_entry.add_person(Person(name), k)
        else:
            if k in bibtex_fields:
                if texify:
                    # the only way i've found to force encoding to latex
                    # not used for now - may be unnecessary with modern bibtex
                    # new_pybtex_entry.fields[k] = unicode(unicode(eblc(v)).encode('latex'))
                    new_pybtex_entry.fields[k] = eblc(v)
                else:
                    new_pybtex_entry.fields[k] = v
    return new_pybtex_entry

def pybtex_to_pieberry(key, ent):
    from pieberry_output import unescape_bad_latex_chars as ublc
    ret = {}
    ret['pdflink'] = ''
    ret['bibtex_key'] = key
    for persons in ent.persons.itervalues():
        for person in persons:
            person.text = unicode(person)
    for k, v in ent.fields.items():
        if k in bibtex_fields:
            ret[k] = v
        if k == 'annote':
            filehere = re_fileloc.search(v)
            if filehere:
                ret['final_fn'] = filehere.group(1)
        if k == 'howpublished' and not ret.has_key('url'):
            urlhere = re_href.search(v)
            if urlhere:
                ret['url'] = ret['pdflink'] = urlhere.group(1)
        if k == 'url':
            ret['pdflink'] = ublc(v)
    formatter = Formatter()
    formatted_names = formatter.format_people(ent)
    rendered_names = formatted_names.render(plaintext.Writer()).rstrip('.')
    corpnamehere = re_corpname.match(rendered_names)
    if corpnamehere:
        ret['corporate_author'] = True
        rendered_names = '%s%s' % (corpnamehere.group(1), corpnamehere.group(2))
    else:
        ret['corporate_author'] = False
    ret['author'] = rendered_names
    ret['pdflink_context'] = ''
    ret['bibtex_type'] = ent.type
    ret['exclude'] = False
    ret['internal_author'] = ''
    ret['internal_title'] = ''
    return ret

def get_top_search_results(searchtext, number=25):
    '''high-level function to return pieberry-friendly list of dicts
    corresponding to top ranked search results in the default
    bibliography'''
    fmts = format_search(searchtext)
    ents = pybtex_entries_from_file(config.get('PBoptions', 'default_bibliography'))
    ranks = exec_search(fmts, ents).items()
    ranks.sort(lambda x, y: cmp(y[1], x[1]))
    return [pybtex_to_pieberry(ky, ents[ky]) for ky, vl in ranks[:number] if vl > 0]

def format_search(text):
    '''formats a text string into a search dict containing bibtex
    field keys and lists of relevant terms to search for in those
    fields.'''
    s = [k for k in re_search.split(text) if k != '']
    search_dict = {}
    if len(s) == 1:
        search_dict['all'] = s[0].split(';')
    for i in range(len(s)):
        if s[i] in search_terms:
            if not s[i] in search_abbrev.values():
                search_dict[search_abbrev[s[i]]] = s[i+1].split(';')
            else:
                search_dict[s[i]] = s[i+1].split(';')
    return search_dict

# weights for search scoring
search_w = {'exact': 35, 'occurrence': 25, 'fuzzy': 10}
field_w = {'key': 30, 'author': 25, 'title': 20, 'other': 10}

def exec_search(searchdict, entries):
    '''search through pybtex entries for terms in a searchdict'''
    # ranked is a dict whose keys are bibtex keys and values are
    # 'scores' for the entries thus indexed.
    def _debug(t):
        # print t
        pass

    ranked = dict([(k, 0) for k in entries.keys()])

    # key exact match algorithm
    if searchdict.has_key('key'):
        for term in searchdict['key']:
            if entries.has_key(term):
                ranked[term] += search_w['exact'] * field_w['key']

    for entkey, entry in entries.items():
        _debug('Entry - %s, %s' % (unicode(entkey), unicode(entry)))
        for serkey, servals in searchdict.items():
            _debug('Search - %s, %s' % (unicode(serkey), unicode(servals)))
            if serkey == 'author' and entry.persons.has_key('author'):
                auth = string.join([unicode(p) for p in entry.persons['author']])
                for term in servals:
                    if term.strip().lower() == auth.strip().lower():
                        ranked[entkey] += search_w['exact'] * field_w['author']
                    elif term.strip().lower() in auth.strip().lower():
                        ranked[entkey] += search_w['occurrence'] * field_w['author']
            elif serkey == 'title' and entry.fields.has_key('title'):
                title = entry.fields['title']
                for term in servals:
                    if term.strip().lower() == title.strip().lower():
                        ranked[entkey] += search_w['exact'] * field_w['title']
                    elif term.strip().lower() in title.strip().lower():
                        ranked[entkey] += search_w['occurrence'] * field_w['title']
            elif serkey == 'key':
                for term in servals:
                    if (term != entkey) and (term.strip().lower() in entkey.strip().lower()):
                        ranked[entkey] += search_w['occurrence'] * field_w['key']
            elif serkey == 'all':
                if entry.persons.has_key('author'):
                    auth = string.join([unicode(p) for p in entry.persons['author']])
                else:
                    auth = ''
                for term in servals:
                    if term.strip().lower() == auth.strip().lower():
                        ranked[entkey] += search_w['exact'] * field_w['author']
                    elif term.strip().lower() in auth.strip().lower():
                        ranked[entkey] += search_w['occurrence'] * field_w['author']
                    for fkey, fval in entry.fields.items():
                        if term.strip().lower() == unicode(fval).strip().lower():
                            if fkey in field_w.keys():
                                ranked[entkey] += search_w['exact'] * field_w[fkey]
                            else:
                                ranked[entkey] += search_w['exact'] * field_w['other']
                        if term.strip().lower() in unicode(fval).strip().lower():
                            if fkey in field_w.keys():
                                ranked[entkey] += search_w['occurrence'] * field_w[fkey]
                            else:
                                ranked[entkey] += search_w['occurrence'] * field_w['other']
            else:
                if entry.fields.has_key(serkey):
                    for term in servals:
                        if term.strip().lower() == unicode(entry.fields[serkey]).strip().lower():
                            ranked[entkey] += search_w['exact'] * field_w['other']
                        elif term.strip().lower() in unicode(entry.fields[serkey]).strip().lower():
                            ranked[entkey] += search_w['occurrence'] * field_w['other']
    return ranked
    
def pybtex_entries_from_file(filename):
    parser = bibtex.Parser()
    bib_data = parser.parse_file(filename)
    return bib_data.entries

def get_formatted_citation(e):
    return get_formatted_citation_html(e)

def get_formatted_citation_from_bibtex(bibtex_entry):
    parser = bibtex.Parser()
    f = open(os.path.join(sysdir, 'temp.bib'), 'w')
    f.write(bibtex_entry)
    f.close()
    bib_data = parser.parse_file(os.path.join(sysdir, 'temp.bib'))
    e = bib_data.entries[bib_data.entries.keys()[0]]
    return get_formatted_citation(e)

# def get_formatted_citation_plaintext(e):
#     # correct for messing with pybtex in an apparently unintended manner
#     for persons in e.persons.itervalues():
#         for person in persons:
#             person.text = unicode(person)
#     formatter = Formatter()
#     formatted_data = formatter.format_booklet_with_url(e)
#     # get hrefs in order
#     rendered_data = formatted_data.render(plaintext.Writer())
#     href_search = re_href.search(rendered_data)
#     output_data = None
#     if href_search:
#         href = href_search.group(1)
#         label = href_search.group(2)
#         output_data = re_href.sub('%s: %s' % (label, href), rendered_data)
#     # hacky reformatting of URLs
#     if not output_data:
#         output_data = rendered_data
#     transtbl = {ord('{'): None, ord('}'): None}
#     from pieberry_output import untexify
#     output_data = untexify(output_data)
#     return output_data
    
# def get_formatted_citation_html(e):
#     # correct for messing with pybtex in an apparently unintended manner
#     for persons in e.persons.itervalues():
#         for person in persons:
#             person.text = unicode(person)
#     formatter = Formatter()
#     formatted_data = formatter.format_booklet(e)
#     rendered_data = formatted_data.render(html.Writer())

#     # get hrefs in order
#     href_search = re_href.search(rendered_data)
#     output_data = None
#     from pieberry_output import unescape_bad_latex_chars as ublc
#     if href_search:
#         href = href_search.group(1)
#         label = href_search.group(2)
#         output_data = re_href.sub('<a href="%s">%s</a>' % (ublc(href), label), rendered_data)
#     else:
#         # hyperlink urls
#         if e.fields.has_key('url'):
#             output_data = rendered_data.replace(e.fields['howpublished'], '<a href="%s">%s</a>' % (ublc(e.fields['url']), e.fields['howpublished']))

#     # hacky reformatting of URLs
#     if not output_data:
#         output_data = rendered_data
#     transtbl = {ord('{'): None, ord('}'): None}
#     output_data = output_data.translate(transtbl)
#     return output_data
        
def get_formatted_citation(e, key='k', format='html'):
    formatter = Formatter()
    ents = ((key, e),)
    formatted_data = formatter.format_entries(ents)
    for d in formatted_data:
        if format == 'html':
            rendered_data = d.text.render(html.Writer())
        else:
            rendered_data = d.text.render(plaintext.Writer())
    # get hrefs in order
    href_search = re_href.search(rendered_data)
    output_data = None
    from pieberry_output import unescape_bad_latex_chars as ublc
    if href_search and format == 'html': 
        href = href_search.group(1)
        label = href_search.group(2)
        output_data = re_href.sub('<a href="%s">%s</a>' % (ublc(href), label), rendered_data)
    elif format == 'html':
        # hyperlink urls
        if e.fields.has_key('url'):
            output_data = rendered_data.replace(e.fields['howpublished'], '<a href="%s">%s</a>' % (ublc(e.fields['url']), e.fields['howpublished']))

    # hacky reformatting of URLs
    if not output_data:
        output_data = rendered_data
    transtbl = {ord('{'): None, ord('}'): None}
    output_data = output_data.translate(transtbl)
    return output_data


if __name__ == '__main__':
    testtext = '''
@Booklet{AEMC2009_DraRepRevNatFraEleDisNetPlaExp,
  author = {{Australian Spingroin}},
  title = {Draft Report - Review of National Framework for Electricity Distribution Network Planning and Expansio},
  howpublished = {\href{http://aemc.gov.au/Media/docs/EPR0015\%20-\%20Draft\%20Report-8959583c-f010-408b-9a52-89dac517020a-0.pdf}{Web site}},
  month = {July},
  year = 2009,
  annote = {Downloaded_on: Wed Jul 14 19:26:55 2010
Publication_date: Tue Jul  7 14:25:00 2009
Saved_to: /home/raif/development/data/Australian Energy Market Commission/Review of National Framework for Electricity Distribution Network Planning and Expansio/20090707 Draft Report.pdf
Author_from_metadata: Rose Campisi
Title_from_metadata: Microsoft Word - DPR Draft Report - Contents and Summary - FINAL FOR PUBLICATION.DOC}}


'''
    import time
    spoofdata = {
        'title': 'Spoof title',
        'author': '{Goldman Saks}',
        'year': '1833',
        'month': 'January',
        'howpublished': "Mungdin{\''u}gn",
        'url': 'http://www.themong.spo.nn/',
        'annote': 'notey',
        'download_time': time.localtime(),
        'creation_date': time.localtime(),
        'final_fn': 'C:noodles',
        'pdflink': 'http://www.hell.no',
        'exclude': True,
        'internal_author': 'Jon Hoo',
        'internal_title': 'Noh Wai',
        'journal': 'journal of stuff',
        'volume': 'Vol 1',
        'bibtex_type': 'article'
        }

    from pprint import pprint
    # pprint.pprint(format_search('author==gronongin, y==1020'))
    # pprint.pprint(format_search('author==gronongin;joney title=wot, y==1020'))
    # pprint.pprint(format_search('lies lies lies'))
    # pprint.pprint(format_search('all==who;wha;there'))
    # st = 'dsfksdf'#'y==1833, all==saks' #'a==saks, key==Sa1833_Spotit'
    # print get_top_search_results(st)

    bibd = pieberry_to_pybtex(spoofdata)
    # print dir(bibd)
    # print bibd.type
    # pprint.pprint(bibd.fields)
    # bibe = pybtex_to_pieberry('thekey', bibd)
    # pprint.pprint(bibe)
    # bib_data = get_formatted_citation_from_bibtex(testtext)
    # bibd = pieberry_to_pybtex(spoofdata)
    bib_data = get_formatted_citation_html_alternate(bibd)
    print bib_data


# a data object from pybtex appears to have as its key attribute a
# dict full of entries referenced by bibtex key, and have additional
# methods 'add_entry', 'add_entries'. These entries in turn have a
# range of attributes including: ('add_person', 'collection',
# 'fields', 'get_crossref', 'persons', 'type', 'vars'). 'persons'
# returns a dict with person objects indexed by bibtex field. 'fields'
# returns a simple dict with bibtex keys and values.

